# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2023 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains the tests of the ledger connection module."""

from unittest import mock
from unittest.mock import ANY, MagicMock, Mock, patch

import pytest
from aea_ledger_ethereum import EthereumCrypto

from aea.common import Address
from aea.crypto.ledger_apis import ETHEREUM_DEFAULT_ADDRESS
from aea.crypto.registries import ledger_apis_registry
from aea.exceptions import AEAException
from aea.helpers.async_utils import AsyncState
from aea.multiplexer import MultiplexerStatus
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue

from packages.valory.connections.ledger.contract_dispatcher import (
    ContractApiRequestDispatcher,
)

# pylint: skip-file
from packages.valory.protocols.contract_api import ContractApiMessage
from packages.valory.protocols.contract_api.dialogues import ContractApiDialogue
from packages.valory.protocols.contract_api.dialogues import (
    ContractApiDialogues as BaseContractApiDialogues,
)


SOME_SKILL_ID = "some/skill:0.1.0"
NON_BLOCKING_TIME = 1
BLOCKING_TIME = 100
TOLERANCE = 1
WAIT_TIME_AMONG_TASKS = 0.1


class ContractApiDialogues(BaseContractApiDialogues):
    """This class keeps track of all contract_api dialogues."""

    def __init__(self, self_address: str) -> None:
        """
        Initialize dialogues.

        :param self_address: the address of the entity for whom dialogues are maintained
        """

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: Address
        ) -> Dialogue.Role:
            """Infer the role of the agent from an incoming/outgoing first message

            :param message: an incoming/outgoing first message
            :param receiver_address: the address of the receiving agent
            :return: The role of the agent
            """
            return ContractApiDialogue.Role.AGENT

        BaseContractApiDialogues.__init__(
            self,
            self_address=self_address,
            role_from_first_message=role_from_first_message,
        )


def test_validate_and_call_callable() -> None:
    """Tests a default method call through ContractApiRequestDispatcher."""

    dummy_address = "0x0000000000000000000000000000000000000000"
    contract_dispatcher = ContractApiRequestDispatcher(
        connection_id=Mock(), connection_state=AsyncState()
    )
    contract = Mock()

    def getAddress(ledger_api=None, contract_address=None, _addr=None):  # type: ignore
        return 0

    contract.getAddress = getAddress
    contract.dummy_method = None
    contract_instance = Mock()
    contract.get_instance.return_value = contract_instance

    ledger_api = ledger_apis_registry.make(
        EthereumCrypto.identifier,
        address=ETHEREUM_DEFAULT_ADDRESS,
    )

    message = MagicMock()
    message.performative = ContractApiMessage.Performative.GET_STATE
    message.kwargs.body = {"_addr": dummy_address}

    message.contract_address = dummy_address

    message.callable = "some"
    contract.some = None
    contract.default_method_call = lambda x, y, z, _addr: 12
    contract.get_state = Mock(side_effect=AttributeError())
    with patch.object(
        contract_dispatcher.contract_registry, "make", return_value=contract
    ):
        assert (
            contract_dispatcher.dispatch_request(
                dialogue=Mock(),
                ledger_api=ledger_api,
                message=message,
                response_builder=lambda x, y: 12,  # type: ignore
            )
            == 12
        )

    contract.some = Mock(side_effect=AEAException("expected"))
    dialogue = Mock()
    dialogue.reply = Mock(return_value=1234)
    with patch.object(
        contract_dispatcher.contract_registry, "make", return_value=contract
    ):
        assert (
            contract_dispatcher.dispatch_request(
                dialogue=dialogue,
                ledger_api=ledger_api,
                message=message,
                response_builder=lambda x, y: 12,  # type: ignore
            )
            == 1234
        )
        dialogue.reply.assert_called_once_with(
            performative=ContractApiMessage.Performative.ERROR,
            target_message=ANY,
            code=500,
            message=ANY,
            data=ANY,
        )

    message.callable = "getAddress"
    # Call a method present in the ABI but not in the contract package
    with mock.patch("web3.contract.contract.ContractFunction.call", return_value=0):
        result = ContractApiRequestDispatcher._validate_and_call_callable(
            ledger_api, message, contract
        )
        assert result == 0

    # Call an non-existent method
    message.callable = "dummy_method"
    contract_instance.get_function_by_name = Mock(side_effect=ValueError())
    with pytest.raises(
        AEAException,
        match=f"Contract method dummy_method not found in ABI of contract {type(contract)}",
    ):
        ContractApiRequestDispatcher._validate_and_call_callable(
            ledger_api, message, contract
        )

    message.performative = None
    message.callable = "getAddress"
    with pytest.raises(AEAException, match="Unexpected performative:"):
        ContractApiRequestDispatcher._validate_and_call_callable(
            ledger_api, message, contract
        )

    message.callable = "some_fn"
    message.performative = ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION
    contract.some_fn = lambda x: None
    with pytest.raises(AEAException, match="Missing required argument `ledger_api`"):
        ContractApiRequestDispatcher._validate_and_call_callable(
            ledger_api, message, contract
        )

    contract.some_fn = lambda: None
    with pytest.raises(
        AEAException, match="Expected one or more positional arguments, got 0"
    ):
        ContractApiRequestDispatcher._validate_and_call_callable(
            ledger_api, message, contract
        )

    contract.some_fn = lambda ledger_api, _addr: None
    ContractApiRequestDispatcher._validate_and_call_callable(
        ledger_api, message, contract
    )

    with pytest.raises(
        AttributeError,
    ):
        ContractApiRequestDispatcher._validate_and_call_callable(
            ledger_api, message, None  # type: ignore
        )


def test_build_response_fails_on_bad_data_type() -> None:
    """Test internal build_response functions for data type check."""
    dispatcher = ContractApiRequestDispatcher(MagicMock(), connection_id="test_id")
    with patch.object(
        dispatcher,
        "dispatch_request",
        lambda x, x1, x2, fn: fn(data=b"some_data", dialogue=MagicMock()),
    ), pytest.raises(
        ValueError, match=r"Invalid state type, got=<class '.+'>, expected=typing.Dict"
    ):
        dispatcher.get_state(MagicMock(), MagicMock(), MagicMock())

    with patch.object(
        dispatcher,
        "dispatch_request",
        lambda x, x1, x2, fn: fn(raw_message=12, dialogue=MagicMock()),
    ), pytest.raises(ValueError, match=r"Invalid message type"):
        dispatcher.get_raw_message(MagicMock(), MagicMock(), MagicMock())

    with patch.object(
        dispatcher,
        "dispatch_request",
        lambda x, x1, x2, fn: fn(transaction=b"some_data", dialogue=MagicMock()),
    ):
        with pytest.raises(
            ValueError,
            match=r"Invalid transaction type, got=<class '.+'>, expected=typing.Dict",
        ):
            dispatcher.get_deploy_transaction(MagicMock(), MagicMock(), MagicMock())
        with pytest.raises(
            ValueError,
            match=r"Invalid transaction type, got=<class '.+'>, expected=typing.Dict",
        ):
            dispatcher.get_raw_transaction(MagicMock(), MagicMock(), MagicMock())


@pytest.mark.asyncio
async def test_run_async() -> None:
    """Test run async error handled."""

    # for pydocstyle
    def _raise():  # type: ignore
        raise Exception("Expected")

    contract_api_dialogues = ContractApiDialogues(SOME_SKILL_ID)
    request, dialogue = contract_api_dialogues.create(
        counterparty="str(ledger_apis_connection.connection_id)",
        performative=ContractApiMessage.Performative.GET_RAW_TRANSACTION,
        ledger_id=EthereumCrypto.identifier,
        contract_id=str(SOME_SKILL_ID),
        contract_address="test addr",
        callable="get_create_batch_transaction",
        kwargs=ContractApiMessage.Kwargs(
            {
                "deployer_address": "test_addr",
                "token_ids": [1, 2, 3, 4, 5, 6, 7, 8, 9, 10],
            }
        ),
    )
    api = None
    msg = await ContractApiRequestDispatcher(
        MultiplexerStatus(), connection_id="test_id"
    ).run_async(
        _raise, api, request, dialogue  # type: ignore
    )
    assert msg.performative == ContractApiMessage.Performative.ERROR  # type: ignore


@pytest.mark.asyncio
async def test_get_handler() -> None:
    """Test failed to get handler."""
    with pytest.raises(Exception, match="Performative not recognized."):
        ContractApiRequestDispatcher(
            MultiplexerStatus(), connection_id="test_id"
        ).get_handler(ContractApiMessage.Performative.ERROR)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Push all available packages to a registry."""


from pathlib import Path
from typing import Dict, Optional, cast

import click

from aea.cli.packages import get_package_manager
from aea.cli.push import push_item_ipfs
from aea.cli.registry.settings import REGISTRY_REMOTE, REMOTE_IPFS
from aea.cli.utils.click_utils import registry_flag
from aea.cli.utils.config import get_default_remote_registry, load_item_config
from aea.configurations.constants import PACKAGES, PYCACHE
from aea.configurations.data_types import PackageId
from aea.package_manager.v0 import PackageManagerV0
from aea.package_manager.v1 import PackageManagerV1


@click.command("push-all")
@click.option(
    "--packages-dir", type=click.Path(file_okay=False, dir_okay=True, exists=True)
)
@click.option(
    "--retries",
    type=int,
    default=1,
    help="Tries on package push to the network.",
)
@registry_flag()
def push_all(packages_dir: Optional[Path], retries: int, registry: str) -> None:
    """Push all available packages to a registry."""
    try:
        push_all_packages(registry, packages_dir, retries=retries)
    except ValueError as e:
        raise click.ClickException(str(e)) from e


def push_all_packages(
    registry: str,
    packages_dir: Optional[Path] = None,
    package_type_config_class: Optional[Dict] = None,
    retries: int = 1,
) -> None:
    """Push all packages."""

    packages: Dict[PackageId, str]

    if registry != REGISTRY_REMOTE:
        raise click.ClickException(
            "Pushing all packages is not supported for the local registry."
        )

    if get_default_remote_registry() != REMOTE_IPFS:
        raise click.ClickException(
            "Pushing all packages is not supported for the HTTP registry."
        )

    if packages_dir is None:
        packages_dir = Path.cwd() / PACKAGES

    package_manager = get_package_manager(
        package_dir=packages_dir,
    )

    if isinstance(package_manager, PackageManagerV0):
        packages = cast(PackageManagerV0, package_manager).packages

    if isinstance(package_manager, PackageManagerV1):
        packages = cast(PackageManagerV1, package_manager).dev_packages

    for package_id in cast(Dict[PackageId, str], packages):
        package_path = package_manager.package_path_from_package_id(
            package_id=package_id,
        )
        if not package_path.is_dir() or package_path.name == PYCACHE:
            continue

        click.echo(f"Pushing: {package_path}")
        item_config = load_item_config(
            item_type=str(package_id.package_type),
            package_path=package_path,
            package_type_config_class=package_type_config_class,
        )
        push_item_ipfs(package_path, item_config.public_id, retries=retries)
        click.echo("")



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the behaviour classes of the erc1155_deploy skill."""
# pylint: skip-file

import logging
from typing import cast
from unittest.mock import patch

from packages.fetchai.protocols.oef_search.message import OefSearchMessage
from packages.fetchai.skills.erc1155_deploy.behaviours import LEDGER_API_ADDRESS
from packages.fetchai.skills.erc1155_deploy.dialogues import ContractApiDialogue
from packages.fetchai.skills.erc1155_deploy.tests.intermediate_class import (
    ERC1155DeployTestCase,
)
from packages.valory.protocols.contract_api.message import ContractApiMessage
from packages.valory.protocols.ledger_api.message import LedgerApiMessage


class TestServiceRegistrationBehaviour(ERC1155DeployTestCase):
    """Test registration behaviour of erc1155_deploy."""

    def test_init(self):
        """Test the __init__ method of the registration behaviour."""
        assert self.registration_behaviour.is_registered is False
        assert self.registration_behaviour.registration_in_progress is False
        assert self.registration_behaviour.failed_registration_msg is None
        assert self.registration_behaviour._nb_retries == 0

    def test_setup(self):
        """Test the setup method of the registration behaviour."""
        # setup
        self.strategy._is_contract_deployed = False

        # before
        assert self.strategy.is_behaviour_active is True

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.registration_behaviour.setup()

        # after
        self.assert_quantity_in_outbox(2)

        # _request_balance
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=LedgerApiMessage,
            performative=LedgerApiMessage.Performative.GET_BALANCE,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.skill_context.skill_id),
            ledger_id=self.strategy.ledger_id,
            address=cast(
                str,
                self.skill.skill_context.agent_addresses.get(self.strategy.ledger_id),
            ),
        )
        assert has_attributes, error_str

        # _request_contract_deploy_transaction
        assert self.strategy.is_behaviour_active is False

        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=ContractApiMessage,
            performative=ContractApiMessage.Performative.GET_DEPLOY_TRANSACTION,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.skill_context.skill_id),
            ledger_id=self.strategy.ledger_id,
            contract_id=self.strategy.contract_id,
            callable="get_deploy_transaction",
            kwargs=ContractApiMessage.Kwargs(
                {
                    "deployer_address": self.skill.skill_context.agent_address,
                    "gas": self.strategy.gas,
                }
            ),
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(
            logging.INFO, "requesting contract deployment transaction..."
        )

    def test_act_i(self):
        """Test the act method of the registration behaviour where failed_registration_msg is NOT None."""
        # setup
        self.registration_behaviour.failed_registration_msg = self.registration_message

        with patch.object(self.logger, "log") as mock_logger:
            self.registration_behaviour.act()

        # after
        self.assert_quantity_in_outbox(1)

        # _retry_failed_registration
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=type(self.registration_message),
            performative=self.registration_message.performative,
            to=self.registration_message.to,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.registration_message.service_description,
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(
            logging.INFO,
            f"Retrying registration on SOEF. Retry {self.registration_behaviour._nb_retries} out of {self.registration_behaviour._max_soef_registration_retries}.",
        )
        assert self.registration_behaviour.failed_registration_msg is None

    def test_act_ii(self):
        """Test the act method of the registration behaviour where failed_registration_msg is NOT None and max retries is reached."""
        # setup
        self.registration_behaviour.failed_registration_msg = self.registration_message
        self.registration_behaviour._max_soef_registration_retries = 2
        self.registration_behaviour._nb_retries = 2

        self.registration_behaviour.act()

        # after
        self.assert_quantity_in_outbox(0)
        assert self.skill.skill_context.is_active is False

    def test_act_iii(self):
        """Test the act method of the registration behaviour where is_behaviour_active IS False."""
        # setup
        self.strategy.is_behaviour_active = False

        # operation
        self.registration_behaviour.act()

        # after
        self.assert_quantity_in_outbox(0)

    def test_act_iv(self):
        """Test the act method of the registration behaviour where is_contract_deployed IS True and is_tokens_created IS False."""
        # setup
        self.strategy.is_contract_deployed = True
        self.strategy._is_tokens_created = False
        self.strategy._contract_address = self.contract_address

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.registration_behaviour.act()

        # after
        self.assert_quantity_in_outbox(1)

        assert self.strategy.is_behaviour_active is False

        # _request_token_create_transaction
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=ContractApiMessage,
            performative=ContractApiMessage.Performative.GET_RAW_TRANSACTION,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.skill_context.skill_id),
            ledger_id=self.strategy.ledger_id,
            contract_id=self.strategy.contract_id,
            contract_address=self.strategy.contract_address,
            callable="get_create_batch_transaction",
            kwargs=ContractApiMessage.Kwargs(
                {
                    "deployer_address": self.skill.skill_context.agent_address,
                    "token_ids": self.strategy.token_ids,
                    "gas": self.strategy.gas,
                }
            ),
        )
        assert has_attributes, error_str

        contract_api_dialogue = cast(
            ContractApiDialogue, self.contract_api_dialogues.get_dialogue(message)
        )
        assert contract_api_dialogue.terms == self.strategy.get_create_token_terms()

        mock_logger.assert_any_call(
            logging.INFO, "requesting create batch transaction..."
        )

    def test_act_v(self):
        """Test the act method of the registration behaviour where is_contract_deployed IS True, is_tokens_created IS True and is_tokens_minted is False."""
        # setup
        self.strategy.is_contract_deployed = True
        self.strategy._is_tokens_created = True
        self.strategy._is_tokens_minted = False
        self.strategy._contract_address = self.contract_address

        # operation
        with patch.object(self.logger, "log") as mock_logger:
            self.registration_behaviour.act()

        # after
        self.assert_quantity_in_outbox(1)

        assert self.strategy.is_behaviour_active is False

        # _request_token_mint_transaction
        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=ContractApiMessage,
            performative=ContractApiMessage.Performative.GET_RAW_TRANSACTION,
            to=LEDGER_API_ADDRESS,
            sender=str(self.skill.skill_context.skill_id),
            ledger_id=self.strategy.ledger_id,
            contract_id=self.strategy.contract_id,
            contract_address=self.strategy.contract_address,
            callable="get_mint_batch_transaction",
            kwargs=ContractApiMessage.Kwargs(
                {
                    "deployer_address": self.skill.skill_context.agent_address,
                    "recipient_address": self.skill.skill_context.agent_address,
                    "token_ids": self.strategy.token_ids,
                    "mint_quantities": self.strategy.mint_quantities,
                    "gas": self.strategy.gas,
                }
            ),
        )
        assert has_attributes, error_str

        contract_api_dialogue = cast(
            ContractApiDialogue, self.contract_api_dialogues.get_dialogue(message)
        )
        assert contract_api_dialogue.terms == self.strategy.get_mint_token_terms()

        mock_logger.assert_any_call(
            logging.INFO, "requesting mint batch transaction..."
        )

    def test_act_vi(self):
        """Test the act method of the registration behaviour where is_contract_deployed IS True, is_tokens_created IS True and is_tokens_minted is True and is_registered IS False."""
        # setup
        self.strategy.is_contract_deployed = True
        self.strategy._is_tokens_created = True
        self.strategy._is_tokens_minted = True
        self.strategy._contract_address = self.contract_address

        # before
        assert self.registration_behaviour.registration_in_progress is False

        # operation
        with patch.object(
            self.strategy,
            "get_location_description",
            return_value=self.mocked_registration_description,
        ) as mock_desc:
            with patch.object(self.logger, "log") as mock_logger:
                self.registration_behaviour.act()

        # after
        self.assert_quantity_in_outbox(1)

        assert self.registration_behaviour.registration_in_progress is True

        # _register_agent
        mock_desc.assert_called_once()

        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.REGISTER_SERVICE,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.mocked_registration_description,
        )
        assert has_attributes, error_str

        mock_logger.assert_any_call(logging.INFO, "registering agent on SOEF.")

    def test_register_service(self):
        """Test the register_service method of the registration behaviour."""
        # operation
        with patch.object(
            self.strategy,
            "get_register_service_description",
            return_value=self.mocked_registration_description,
        ):
            with patch.object(self.logger, "log") as mock_logger:
                self.registration_behaviour.register_service()

        # after
        self.assert_quantity_in_outbox(1)

        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.REGISTER_SERVICE,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.mocked_registration_description,
        )
        assert has_attributes, error_str
        mock_logger.assert_any_call(
            logging.INFO, "registering agent's service on the SOEF."
        )

    def test_register_genus(self):
        """Test the register_genus method of the registration behaviour."""
        # operation
        with patch.object(
            self.strategy,
            "get_register_personality_description",
            return_value=self.mocked_registration_description,
        ):
            with patch.object(self.logger, "log") as mock_logger:
                self.registration_behaviour.register_genus()

        # after
        self.assert_quantity_in_outbox(1)

        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.REGISTER_SERVICE,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.mocked_registration_description,
        )
        assert has_attributes, error_str
        mock_logger.assert_any_call(
            logging.INFO, "registering agent's personality genus on the SOEF."
        )

    def test_register_classification(self):
        """Test the register_classification method of the registration behaviour."""
        # operation
        with patch.object(
            self.strategy,
            "get_register_classification_description",
            return_value=self.mocked_registration_description,
        ):
            with patch.object(self.logger, "log") as mock_logger:
                self.registration_behaviour.register_classification()

        # after
        self.assert_quantity_in_outbox(1)

        message = self.get_message_from_outbox()
        has_attributes, error_str = self.message_has_attributes(
            actual_message=message,
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.REGISTER_SERVICE,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.mocked_registration_description,
        )
        assert has_attributes, error_str
        mock_logger.assert_any_call(
            logging.INFO, "registering agent's personality classification on the SOEF."
        )

    def test_teardown(self):
        """Test the teardown method of the service_registration behaviour."""
        # operation
        with patch.object(
            self.strategy,
            "get_unregister_service_description",
            return_value=self.mocked_registration_description,
        ):
            with patch.object(
                self.strategy,
                "get_location_description",
                return_value=self.mocked_registration_description,
            ):
                with patch.object(self.logger, "log") as mock_logger:
                    self.registration_behaviour.teardown()

        # after
        self.assert_quantity_in_outbox(2)

        # _unregister_service
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.UNREGISTER_SERVICE,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.mocked_registration_description,
        )
        assert has_attributes, error_str
        mock_logger.assert_any_call(logging.INFO, "unregistering service from SOEF.")

        # _unregister_agent
        has_attributes, error_str = self.message_has_attributes(
            actual_message=self.get_message_from_outbox(),
            message_type=OefSearchMessage,
            performative=OefSearchMessage.Performative.UNREGISTER_SERVICE,
            to=self.skill.skill_context.search_service_address,
            sender=str(self.skill.skill_context.skill_id),
            service_description=self.mocked_registration_description,
        )
        assert has_attributes, error_str
        mock_logger.assert_any_call(logging.INFO, "unregistering agent from SOEF.")



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests of the dialogue classes of the ml_train skill."""
# pylint: skip-file

from aea.test_tools.test_skill import COUNTERPARTY_AGENT_ADDRESS

from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.protocols.gym.message import GymMessage
from packages.fetchai.skills.gym.dialogues import DefaultDialogue, GymDialogue
from packages.fetchai.skills.gym.tests.intermediate_class import GymTestCase


class TestDialogues(GymTestCase):
    """Test dialogue classes of gym."""

    def test_default_dialogues(self):
        """Test the DefaultDialogues class."""
        _, dialogue = self.default_dialogues.create(
            counterparty=COUNTERPARTY_AGENT_ADDRESS,
            performative=DefaultMessage.Performative.BYTES,
            content=self.content_bytes,
        )
        assert dialogue.role == DefaultDialogue.Role.AGENT
        assert dialogue.self_address == str(self.skill.skill_context.skill_id)

    def test_gym_dialogues(self):
        """Test the GymDialogues class."""
        _, dialogue = self.gym_dialogues.create(
            counterparty=COUNTERPARTY_AGENT_ADDRESS,
            performative=GymMessage.Performative.RESET,
        )
        assert dialogue.role == GymDialogue.Role.AGENT
        assert dialogue.self_address == str(self.skill.skill_context.skill_id)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Memory usage check."""
import time
from threading import Thread
from typing import List, Tuple, Union

from aea_cli_benchmark.utils import SyncedGeneratorConnection  # noqa: I100
from aea_cli_benchmark.utils import (
    get_mem_usage_in_mb,
    make_agent,
    make_envelope,
    make_skill,
    wait_for_condition,
)

from aea.protocols.base import Message
from aea.registries.resources import Resources
from aea.skills.base import Handler

from packages.fetchai.protocols.default.message import DefaultMessage


class TestHandler(Handler):
    """Dummy handler to handle messages."""

    SUPPORTED_PROTOCOL = DefaultMessage.protocol_id

    def setup(self) -> None:
        """Noop setup."""

    def teardown(self) -> None:
        """Noop teardown."""

    def handle(self, message: Message) -> None:
        """Handle incoming message."""
        self.context.outbox.put(make_envelope(message.to, message.sender))


def run(duration: int, runtime_mode: str) -> List[Tuple[str, Union[int, float]]]:
    """Check memory usage."""
    # pylint: disable=import-outside-toplevel,unused-import
    # import manually due to some lazy imports in decision_maker
    import aea.decision_maker.default  # noqa: F401

    connection = SyncedGeneratorConnection.make()
    resources = Resources()
    resources.add_connection(connection)
    agent = make_agent(runtime_mode=runtime_mode, resources=resources)
    agent.resources.add_skill(make_skill(agent, handlers={"test": TestHandler}))

    t = Thread(target=agent.start, daemon=True)
    t.start()
    wait_for_condition(lambda: agent.is_running, timeout=5)

    connection.enable()
    time.sleep(duration)
    connection.disable()
    mem_usage = get_mem_usage_in_mb()
    agent.stop()
    t.join(5)
    rate = connection.count_in / duration

    return [
        ("envelopes received", connection.count_in),
        ("envelopes sent", connection.count_out),
        ("rate (envelopes/second)", rate),
        ("mem usage (Mb)", mem_usage),
    ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test `generate-all-protocols` command."""

import logging
import pprint
import shutil
import subprocess  # nosec
from pathlib import Path
from typing import Any
from unittest import mock

import pytest

from aea.configurations.constants import (
    DEFAULT_PROTOCOL_CONFIG_FILE,
    DEFAULT_README_FILE,
)
from aea.exceptions import enforce
from aea.manager.helpers import AEAProject
from aea.test_tools.exceptions import AEATestingException
from aea.test_tools.test_cases import AEATestCaseMany


def _run_cli_patch(*args: Any, **kwargs: Any) -> None:
    if "isort" in args or "black" in args:
        return

    print(f"Calling command {args} with kwargs {kwargs}")
    return_code = subprocess.check_call(args, **kwargs)  # nosec
    enforce(
        return_code == 0,
        f"Return code of {pprint.pformat(args)} is {return_code} != 0.",
    )


class BaseGenerateAllProtocolsTestCase(AEATestCaseMany):
    """Base test class."""

    use_packages_dir: bool = True
    find_packages_patch: Any
    package_path: Path

    @classmethod
    def setup_class(cls) -> None:
        """Setup test class."""
        super().setup_class()

        cls.packages_dir_path = cls.packages_dir_path.absolute()
        cls.package_path = cls.packages_dir_path / "fetchai" / "protocols" / "fipa"
        cls.find_packages_patch = mock.patch(
            "aea.cli.generate_all_protocols.find_protocols_in_local_registry",
            return_value=(cls.package_path,),
        )


class TestGenerateAllProtcols(BaseGenerateAllProtocolsTestCase):
    """Test generate all protocols."""

    def test_run(self, caplog) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ), caplog.at_level(logging.INFO):
            result = self.run_cli_command(
                "generate-all-protocols",
                str(self.packages_dir_path),
                "--root-dir",
                str(self.packages_dir_path.parent),
            )

            assert result.exit_code == 0
            assert f"Processing protocol at path {self.package_path}"

    def test_check_clean_pass(self) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ), mock.patch(
            "aea.cli.generate_all_protocols.check_working_tree_is_dirty",
            return_value=True,
        ):
            result = self.run_cli_command(
                "generate-all-protocols",
                str(self.packages_dir_path),
                "--root-dir",
                str(self.packages_dir_path.parent),
                "--check-clean",
            )

            assert result.exit_code == 0, result.output

    def test_check_clean_fail(self) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ), mock.patch(
            "aea.cli.generate_all_protocols.check_working_tree_is_dirty",
            return_value=False,
        ):
            with pytest.raises(AEATestingException, match="Exit code: 1"):
                self.run_cli_command(
                    "generate-all-protocols",
                    str(self.packages_dir_path),
                    "--root-dir",
                    str(self.packages_dir_path.parent),
                    "--check-clean",
                )

    def test_check_bump(self) -> None:
        """Test command invocation."""

        fipa_dir = self.package_path

        def _download_patch(_: Any, temp_dir: str):
            temp_fipa = Path(temp_dir, "fipa")
            shutil.copytree(fipa_dir, temp_fipa)

            config_file = temp_fipa / DEFAULT_PROTOCOL_CONFIG_FILE
            config_file.write_text(
                config_file.read_text().replace("version: 1.0.0", "version: 0.1.0")
            )

            readme_file = temp_fipa / DEFAULT_README_FILE
            readme_file.write_text(
                readme_file.read_text().replace("version: 1.0.0", "version: 0.1.0")
            )

        with self.find_packages_patch, mock.patch(
            "aea.cli.generate_all_protocols.download_package", new=_download_patch
        ), mock.patch(
            "aea.cli.generate_all_protocols._process_packages_protocol"
        ), mock.patch(
            "aea.cli.generate_all_protocols.log"
        ) as logger:
            self.run_cli_command(
                "generate-all-protocols",
                str(self.packages_dir_path),
                "--root-dir",
                str(self.packages_dir_path.parent),
            )
            logger.assert_called_with(
                "Bumping protocol specification id from 'fetchai/fipa:1.0.0' to 'fetchai/fipa:1.0.0'"
            )

    def test_check_bump_fail(self) -> None:
        """Test command invocation."""

        fipa_dir = self.package_path

        def _download_patch(_: Any, temp_dir: str):
            temp_fipa = Path(temp_dir, "fipa")
            shutil.copytree(fipa_dir, temp_fipa)

        with self.find_packages_patch, mock.patch(
            "aea.cli.generate_all_protocols.download_package", new=_download_patch
        ), mock.patch(
            "aea.cli.generate_all_protocols._process_packages_protocol"
        ), mock.patch(
            "aea.cli.generate_all_protocols.log"
        ) as logger:
            self.run_cli_command(
                "generate-all-protocols",
                str(self.packages_dir_path),
                "--root-dir",
                str(self.packages_dir_path.parent),
            )
            logger.assert_called_with(
                "Protocol specification id not bumped - content is not different, or version is not newer."
            )

    def test_no_bump(self) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ):
            self.run_cli_command(
                "generate-all-protocols",
                str(self.packages_dir_path),
                "--root-dir",
                str(self.packages_dir_path.parent),
                "--no-bump",
            )

    def test_no_bump_failure(self) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ), mock.patch("re.compile", return_value=mock.MagicMock(search=lambda _: None)):
            with pytest.raises(
                ValueError, match="protocol generator docstring not found"
            ):
                self.run_cli_command(
                    "generate-all-protocols",
                    str(self.packages_dir_path),
                    "--root-dir",
                    str(self.packages_dir_path.parent),
                    "--no-bump",
                )


class TestParentAsRootDir(BaseGenerateAllProtocolsTestCase):
    """Test generate all protocols."""

    def test_root_dir_parent(self, caplog) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ), caplog.at_level(logging.INFO):
            result = self.run_cli_command(
                "generate-all-protocols",
                str(self.packages_dir_path),
                "--root-dir",
                str(self.packages_dir_path.parent.parent),
            )

            assert result.exit_code == 0
            assert (
                "Replace prefix of import statements in directory 'fipa'" in caplog.text
            )

    def test_root_dir_dont_match(self) -> None:
        """Test command invocation."""

        with self.find_packages_patch, mock.patch.object(
            AEAProject, "run_cli", new=_run_cli_patch
        ):
            with pytest.raises(
                ValueError,
                match="Packages dir should be a sub directory of the root directory.",
            ):
                self.run_cli_command(
                    "generate-all-protocols",
                    str(self.packages_dir_path),
                )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains a test for aea.test_tools.test_cases."""

import logging
import os
import shutil
import tempfile
import time
from pathlib import Path
from unittest import TestCase, mock

import pytest
import yaml
from aea_ledger_ethereum import EthereumCrypto
from aea_ledger_fetchai import FetchAICrypto

import aea
from aea.configurations.base import AgentConfig
from aea.configurations.constants import (
    DEFAULT_AEA_CONFIG_FILE,
    PRIVATE_KEY_PATH_SCHEMA,
)
from aea.configurations.manager import AgentConfigManager
from aea.mail.base import Envelope
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue
from aea.test_tools.exceptions import AEATestingException
from aea.test_tools.test_cases import (
    AEATestCase,
    AEATestCaseEmpty,
    AEATestCaseEmptyFlaky,
    AEATestCaseManyFlaky,
    BaseAEATestCase,
)
from aea.test_tools.test_contract import BaseContractTestCase

from packages.fetchai.connections.stub.connection import PUBLIC_ID as STUB_CONNECTION_ID
from packages.fetchai.protocols.default.dialogues import (
    DefaultDialogue,
    DefaultDialogues,
)
from packages.fetchai.protocols.default.message import DefaultMessage
from packages.fetchai.skills.echo import PUBLIC_ID as ECHO_SKILL_PUBLIC_ID
from packages.fetchai.skills.error import PUBLIC_ID as ERROR_SKILL_PUBLIC_ID

from tests.conftest import MAX_FLAKY_RERUNS, MY_FIRST_AEA_PUBLIC_ID
from tests.data import dummy_aea
from tests.test_cli import test_generate_wealth


TestWealthCommandsPositive = test_generate_wealth.TestWealthCommandsPositive


class TestConfigCases(AEATestCaseEmpty):
    """Test config set/get."""

    @classmethod
    def setup_class(cls):
        """Setup class."""
        super(TestConfigCases, cls).setup_class()
        cls.add_item("connection", str(STUB_CONNECTION_ID))
        cls.add_item("skill", str(ERROR_SKILL_PUBLIC_ID))

    def test_agent_nested_set_agent_crudcollection(self):
        """Test agent test nested set from path."""
        key_name = "agent.private_key_paths.cosmos"
        self.nested_set_config(key_name, "testdata2000")
        result = self.run_cli_command("config", "get", key_name, cwd=self._get_cwd())
        assert b"testdata2000" in result.stdout_bytes

    def test_agent_nested_set_agent_crudcollection_all(self):
        """Test agent test nested set from path."""
        key_name = "agent.private_key_paths"
        self.nested_set_config(key_name, {"cosmos": "testdata2000"})
        result = self.run_cli_command(
            "config", "get", f"{key_name}.cosmos", cwd=self._get_cwd()
        )
        assert b"testdata2000" in result.stdout_bytes

    def test_agent_nested_set_agent_simple(self):
        """Test agent test nested set from path."""
        key_name = "agent.default_ledger"
        self.nested_set_config(key_name, "some_ledger")
        result = self.run_cli_command("config", "get", key_name, cwd=self._get_cwd())
        assert b"some_ledger" in result.stdout_bytes

    def test_agent_nested_set_skill_simple(self):
        """Test agent test nested set from path."""
        key_name = "vendor.fetchai.skills.error.handlers.error_handler.args.some_key"
        self.nested_set_config(key_name, "some_value")
        result = self.run_cli_command("config", "get", key_name, cwd=self._get_cwd())
        assert b"some_value" in result.stdout_bytes

    def test_agent_nested_set_skill_simple_nested(self):
        """Test agent test nested set from path."""
        key_name = "vendor.fetchai.skills.error.handlers.error_handler.args.some_key"
        self.nested_set_config(f"{key_name}.some_nested_key", "some_value")

    def test_agent_nested_set_skill_all(self):
        """Test agent test nested set from path."""
        key_name = "vendor.fetchai.skills.error.handlers.error_handler.args"
        self.nested_set_config(key_name, {"some_key": "some_value"})
        result = self.run_cli_command(
            "config", "get", f"{key_name}.some_key", cwd=self._get_cwd()
        )
        assert b"some_value" in result.stdout_bytes

    def test_agent_nested_set_skill_all_nested(self):
        """Test agent test nested set from path."""
        key_name = "vendor.fetchai.skills.error.handlers.error_handler.args"
        self.nested_set_config(
            key_name, {"some_key": {"some_nested_key": "some_value"}}
        )

    def test_agent_nested_set_connection_simple(self):
        """Test agent test nested set from path."""
        key_name = "vendor.fetchai.connections.stub.config.input_file"
        self.nested_set_config(key_name, "some_value")
        result = self.run_cli_command("config", "get", key_name, cwd=self._get_cwd())
        assert b"some_value" in result.stdout_bytes

    def test_agent_nested_set_connection_dependency(self):
        """Test agent test nested set from path."""
        key_name = "vendor.fetchai.connections.stub.dependencies"
        self.nested_set_config(key_name, {"dep": {"version": "==1.0.0"}})

    def test_agent_set(self):
        """Test agent test set from path."""
        value = True
        key_name = "agent.logging_config.disable_existing_loggers"
        self.set_config(key_name, value, aev=True)
        result = self.run_cli_command("config", "get", key_name, cwd=self._get_cwd())
        assert str(value) in str(result.stdout_bytes)

    def test_agent_get_exception(self):
        """Test agent test get non exists key."""
        with pytest.raises(Exception, match=".*bad_key.*"):
            self.run_cli_command("config", "get", "agent.bad_key", cwd=self._get_cwd())


class TestConfigCasesAEV(AEATestCaseEmpty):
    """Test configuration overwrite via CLI tool"""

    def test_agent_set_aev(self):
        """Test agent skill overwrite using environment variable."""

        def copy_dummy_aea_and_update_aea_config_with_env_var():
            shutil.copytree(dummy_aea.__path__[0], agent_dir)
            aea_config_file = agent_dir / "aea-config.yaml"
            content = aea_config_file.read_text()
            new_content = content + f"\nis_abstract: ${{{env_var}:bool:false}}"
            aea_config_file.write_text(new_content)

        def set_config_load_agent_return_component_config():
            self.set_config(key_name, value, aev=True)
            manager = AgentConfigManager.load(agent_dir, substitude_env_vars=True)
            return manager.agent_config.json["component_configurations"]

        value = True
        key_name = "agent.logging_config.disable_existing_loggers"
        env_var = "DUMMY_ENV_VAR"
        assert env_var not in os.environ

        with tempfile.TemporaryDirectory() as tmp_dir:
            agent_dir = Path(tmp_dir, "dummy_aea")
            copy_dummy_aea_and_update_aea_config_with_env_var()
            try:
                component_configs = set_config_load_agent_return_component_config()
                assert not any(c["is_abstract"] for c in component_configs)
                os.environ[env_var] = "true"
                component_configs = set_config_load_agent_return_component_config()
                assert any(c["is_abstract"] for c in component_configs)
            finally:
                os.environ.pop(env_var, None)


class TestRunAgent(AEATestCaseEmpty):
    """Tests test for generic cases of AEATestCases."""

    def test_run_agent(self):
        """Run agent and test it's launched."""
        self.generate_private_key()
        self.add_private_key()
        process = self.run_agent()
        assert self.is_running(process, timeout=30)

        with mock.patch("platform.system", return_value="TempleOS"):
            with pytest.raises(RuntimeError, match="Platform TempleOS not supported"):
                self.terminate_agents()

        with mock.patch("subprocess.Popen.poll", return_value=None):
            with pytest.raises(TimeoutError, match="Failed to terminate agents"):
                self.terminate_agents()


class TestTeardownClassTimeout:
    """Test BaseAEATestCase.teardown_class timeout"""

    def setup(self):
        """Setup test"""
        BaseAEATestCase.setup_class()
        self.test = BaseAEATestCase()

    def test_teardown_class_timeout(self, caplog):
        """Test teardown_class timeout"""

        with mock.patch.object(
            BaseAEATestCase, "is_successfully_terminated", return_value=False
        ):
            with mock.patch("time.sleep", return_value=None):
                with caplog.at_level(logging.ERROR):
                    self.test.teardown_class()
                    assert "Not all subprocesses terminated successfully" in caplog.text
                    assert "Non-zero returncodes" in caplog.text


class TestGenericCases(AEATestCaseEmpty):
    """Tests test for generic cases of AEATestCases."""

    def test_disable_aea_logging(self):
        """Call logging disable."""
        self.disable_aea_logging()

    def test_run_install(self):
        """Test run_install"""
        result = self.run_install()
        assert result.exit_code == 0

    def test_start_subprocess(self):
        """Start a python subprocess and check output."""
        proc = self.start_subprocess("-c", "print('hi')")
        proc.wait(10)

        assert "hi" in self.stdout[proc.pid]

    def test_start_thread(self):
        """Start and join thread for python code."""
        called = False

        def fn():
            nonlocal called
            called = True

        thread = self.start_thread(fn)
        thread.join()
        assert called

    def test_fetch_and_delete(self):
        """Fetch and delete agent from repo."""
        agent_name = "some_agent_for_tests"
        self.fetch_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        assert os.path.exists(agent_name)
        self.delete_agents(agent_name)
        assert not os.path.exists(agent_name)

    def test_diff(self):
        """Test difference_to_fetched_agent."""
        agent_name = "some_agent_for_tests2"
        self.fetch_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        self.run_cli_command(
            "config", "set", "agent.default_ledger", "test_ledger", cwd=agent_name
        )
        result = self.run_cli_command(
            "config", "get", "agent.default_ledger", cwd=agent_name
        )
        assert b"test_ledger" in result.stdout_bytes
        diff = self.difference_to_fetched_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        assert diff
        assert "test_ledger" in diff[1]

    def test_no_diff(self):
        """Test no difference for two aea configs."""
        agent_name = "some_agent_for_tests3"
        self.fetch_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        diff = self.difference_to_fetched_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        assert not diff

    def test_diff_different_overrides(self):
        """Test difference due to overrides."""
        agent_name = "some_agent_for_tests4"
        self.fetch_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        self.set_agent_context(agent_name)
        self.run_cli_command(
            "config",
            "set",
            "vendor.fetchai.skills.echo.behaviours.echo.args.tick_interval",
            "2.0",
            cwd=self._get_cwd(),
        )
        diff = self.difference_to_fetched_agent(str(MY_FIRST_AEA_PUBLIC_ID), agent_name)
        assert diff
        assert diff[0] == DEFAULT_AEA_CONFIG_FILE

    def test_terminate_subprocesses(self):
        """Start and terminate long running python subprocess."""
        proc = self.start_subprocess("-c", "import time; time.sleep(10)")
        assert proc.returncode is None
        self._terminate_subprocesses()
        self.subprocesses.clear()
        assert proc.returncode is not None

    def test_miss_from_output(self):
        """Test subprocess output missing output."""
        proc = self.start_subprocess("-c", "print('hi')")
        assert len(self.missing_from_output(proc, ["hi"], timeout=5)) == 0
        assert "HI" in self.missing_from_output(proc, ["HI"], timeout=5)

    def test_replace_file_content(self):
        """Replace content of the file with another one."""
        file1 = "file1.txt"
        file2 = "file2.txt"

        with open(file1, "w") as f:
            f.write("hi")

        with open(file2, "w") as f:
            f.write("world")

        self.replace_file_content(Path(file1), Path(file2))

        with open(file2, "r") as f:
            assert f.read() == "hi"


class TestDifferenceToFetchedAgent(BaseAEATestCase):
    """Test 'different_to_fetched_agent' in case of component overrides."""

    _mock_called = False
    original_function = yaml.safe_load_all
    test_agent_name: str

    @classmethod
    def setup_class(cls) -> None:
        """Set up the class."""
        super().setup_class()

        # build aea, and override the tick interval
        cls.test_agent_name = "test_agent"
        cls.fetch_agent(str(MY_FIRST_AEA_PUBLIC_ID), cls.test_agent_name)
        cls.set_agent_context(cls.test_agent_name)
        cls.run_cli_command(
            "config",
            "set",
            "vendor.fetchai.skills.echo.behaviours.echo.args.tick_interval",
            "2.0",
            cwd=cls._get_cwd(),
        )

    @classmethod
    def _safe_load_all_side_effect(cls, file):
        """Implement yaml.safe_load_all side-effect for testing."""
        result = list(cls.original_function(file))
        if not cls._mock_called:
            cls._mock_called = True
            fake_override = {}
            result.append(fake_override)
        return iter(result)

    def test_difference_to_fetched_agent(self, *_mocks):
        """Test difference to fetched agent."""
        with mock.patch(
            "yaml.safe_load_all", side_effect=self._safe_load_all_side_effect
        ):
            file_diff = self.difference_to_fetched_agent(
                str(MY_FIRST_AEA_PUBLIC_ID), self.test_agent_name
            )
            assert file_diff


class TestLoadAgentConfig(AEATestCaseEmpty):
    """Test function 'load_agent_config'."""

    def test_load_agent_config(self):
        """Test load_agent_config."""
        agent_config = self.load_agent_config(self.agent_name)
        assert isinstance(agent_config, AgentConfig)

    def test_load_agent_config_when_agent_name_not_exists(self):
        """Test load_agent_config with a wrong agent name."""
        wrong_agent_name = "non-existing-agent-name"
        with pytest.raises(
            AEATestingException,
            match=f"Cannot find agent '{wrong_agent_name}' in the current test case.",
        ):
            self.load_agent_config(wrong_agent_name)


class TestAddAndEjectComponent(AEATestCaseEmpty):
    """Test add/reject components."""

    def test_add_and_eject(self):
        """Test add/reject components."""
        result = self.add_item("skill", str(ECHO_SKILL_PUBLIC_ID), local=True)
        assert result.exit_code == 0

        result = self.eject_item("skill", str(ECHO_SKILL_PUBLIC_ID))
        assert result.exit_code == 0


class TestAddAndRemoveComponent(AEATestCaseEmpty):
    """Test add/remove components."""

    def test_add_and_eject(self):
        """Test add/reject components."""
        result = self.add_item("skill", str(ECHO_SKILL_PUBLIC_ID), local=True)
        assert result.exit_code == 0

        result = self.remove_item("skill", str(ECHO_SKILL_PUBLIC_ID))
        assert result.exit_code == 0


class TestGenerateAndAddKey(AEATestCaseEmpty):
    """Test generate and add private key."""

    def test_generate_and_add_key(self):
        """Test generate and add private key."""
        result = self.generate_private_key("cosmos")
        assert result.exit_code == 0
        result = self.add_private_key(
            "cosmos", "cosmos_private_key.txt", connection=True
        )
        assert result.exit_code == 0
        result = self.add_private_key("cosmos", "cosmos_private_key.txt")
        assert result.exit_code == 0

        result = self.remove_private_key("cosmos")
        assert result.exit_code == 0


class TestGetWealth(AEATestCaseEmpty):
    """Test get_wealth."""

    @pytest.mark.flaky(reruns=MAX_FLAKY_RERUNS)
    def test_get_wealth(self):
        """Test get_wealth."""
        # just call it, network related and quite unstable
        self.generate_private_key(FetchAICrypto.identifier)
        self.add_private_key(
            FetchAICrypto.identifier,
            PRIVATE_KEY_PATH_SCHEMA.format(FetchAICrypto.identifier),
        )
        self.get_wealth(FetchAICrypto.identifier)


class TestGetAddress(AEATestCaseEmpty):
    """Test get_address."""

    def test_get_address(self):
        """Test get_address."""
        # just call it, network related and quite unstable
        self.generate_private_key()
        self.add_private_key()
        result = self.get_address(EthereumCrypto.identifier)
        assert len(result) == 42
        assert result.startswith("0x")


class TestAEA(AEATestCase):
    """Test agent test set from path."""

    path_to_aea = Path("tests") / "data" / "dummy_aea"

    def test_scaffold_and_fingerprint(self):
        """Test component scaffold and fingerprint."""
        result = self.scaffold_item("skill", "skill1")
        assert result.exit_code == 0

        result = self.fingerprint_item("skill", "fetchai/skill1:0.1.0")
        assert result.exit_code == 0

    def test_scaffold_and_fingerprint_protocol(self):
        """Test component scaffold and fingerprint protocol."""
        result = self.scaffold_item("protocol", "protocol1")
        assert result.exit_code == 0

        result = self.fingerprint_item("protocol", "fetchai/protocol1:0.1.0")
        assert result.exit_code == 0


class TestSendReceiveEnvelopesSkill(AEATestCaseEmpty):
    """Test that we can communicate with agent via stub connection."""

    def test_send_receive_envelope(self):
        """Run the echo skill sequence."""
        self.generate_private_key()
        self.add_private_key()
        self.add_item("connection", str(STUB_CONNECTION_ID))
        self.add_item("skill", str(ECHO_SKILL_PUBLIC_ID))

        process = self.run_agent()
        is_running = self.is_running(process)
        assert is_running, "AEA not running within timeout!"

        # add sending and receiving envelope from input/output files
        sender = "sender"

        def role_from_first_message(  # pylint: disable=unused-argument
            message: Message, receiver_address: str
        ) -> Dialogue.Role:
            return DefaultDialogue.Role.AGENT

        default_dialogues = DefaultDialogues(sender, role_from_first_message)
        message_content = b"hello"
        message = DefaultMessage(
            performative=DefaultMessage.Performative.BYTES,
            dialogue_reference=default_dialogues.new_self_initiated_dialogue_reference(),
            content=message_content,
        )
        sent_envelope = Envelope(
            to=self.agent_name,
            sender=sender,
            protocol_specification_id=message.protocol_specification_id,
            message=message,
        )

        self.send_envelope_to_agent(sent_envelope, self.agent_name)

        time.sleep(2.0)
        received_envelope = self.read_envelope_from_agent(self.agent_name)
        received_message = DefaultMessage.serializer.decode(received_envelope.message)
        assert sent_envelope.message.content == received_message.content


class TestInvoke(AEATestCaseEmpty):
    """Test invoke method."""

    def test_invoke(self):
        """Test invoke method."""
        result = self.invoke("--version")
        assert result.exit_code == 0
        assert f"aea, version {aea.__version__}" in result.stdout


class TestFlakyMany(AEATestCaseManyFlaky):
    """Test that flaky tests are properly rerun."""

    @pytest.mark.flaky(reruns=1)
    def test_fail_on_first_run(self):
        """Test failure on first run leads to second run."""
        file = os.path.join(self.t, "test_file")
        if self.run_count == 1:
            open(file, "a").close()
            raise AssertionError("Expected error to trigger rerun!")
        assert self.run_count == 2, "Should only be rerun once!"
        assert not os.path.isfile(file), "File should not exist"


class TestFlakyEmpty(AEATestCaseEmptyFlaky):
    """Test that flaky tests are properly rerun."""

    @pytest.mark.flaky(reruns=1)
    def test_fail_on_first_run(self):
        """Test failure on first run leads to second run."""
        file = os.path.join(self.t, "test_file")
        if self.run_count == 1:
            open(file, "a").close()
            raise AssertionError("Expected error to trigger rerun!")
        assert self.run_count == 2, "Should only be rerun once!"
        assert not os.path.isfile(file), "File should not exist"


class TestBaseContractTestCase(TestCase):
    """Test case for BaseContractTestCase ABC class."""

    @mock.patch(
        "aea.test_tools.test_contract.BaseContractTestCase.sign_send_confirm_receipt_multisig_transaction"
    )
    def test_sign_send_confirm_receipt_transaction(
        self, sign_send_confirm_receipt_multisig_transaction_mock
    ):
        """Test sign_send_confirm_receipt_multisig_transaction is called for backward compatibility."""

        class ContractTestCase(BaseContractTestCase):
            path_to_contract = Path(".")
            ledger_identifier = ""

        ContractTestCase.sign_send_confirm_receipt_transaction(
            "tx", "ledger_api", "crypto"
        )
        sign_send_confirm_receipt_multisig_transaction_mock.assert_called_once()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of the 'aea push' subcommand."""
import os
from pathlib import Path
from shutil import copytree
from typing import Dict, Optional, Union, cast

import click
from click.exceptions import ClickException

from aea.cli.registry.push import check_package_public_id, push_item
from aea.cli.registry.settings import REGISTRY_LOCAL, REMOTE_HTTP, REMOTE_IPFS
from aea.cli.utils.click_utils import (
    PublicIdOrPathParameter,
    component_flag,
    registry_flag,
)
from aea.cli.utils.config import (
    get_default_remote_registry,
    get_ipfs_node_multiaddr,
    get_registry_path_from_cli_config,
    load_item_config,
)
from aea.cli.utils.context import Context
from aea.cli.utils.decorators import check_aea_project
from aea.cli.utils.package_utils import (
    try_get_item_source_path,
    try_get_item_target_path,
)
from aea.configurations.data_types import PublicId
from aea.helpers.cid import to_v1


try:
    from aea_cli_ipfs.core import IPFSTool  # type: ignore

    IS_IPFS_PLUGIN_INSTALLED = True
except ImportError:
    IS_IPFS_PLUGIN_INSTALLED = False


@click.command()
@registry_flag()
@component_flag(wrap_public_id=False)
@click.argument("public_id_or_path", type=PublicIdOrPathParameter())
@click.pass_context
def push(
    click_context: click.Context,
    component_type: str,
    public_id_or_path: Union[PublicId, Path],
    registry: str,
) -> None:
    """Push a non-vendor package of the agent to the registry."""

    if isinstance(public_id_or_path, PublicId):
        push_from_public_id(
            click_context, component_type, cast(PublicId, public_id_or_path), registry
        )
    else:
        push_item_from_path(
            click_context, component_type, cast(Path, public_id_or_path), registry
        )


@check_aea_project
def push_from_public_id(
    context: click.Context, component_type: str, public_id: PublicId, registry: str
) -> None:
    """Push from public id."""
    ctx = cast(Context, context.obj)
    if registry == REGISTRY_LOCAL:
        _save_item_locally(ctx, component_type, public_id)
    else:
        if get_default_remote_registry() == REMOTE_IPFS:
            raise click.ClickException(
                "IPFS registry not supported while pushing an item using public id."
            )

        push_item(ctx, component_type, public_id)


def _save_item_locally(ctx: Context, item_type: str, item_id: PublicId) -> None:
    """
    Save item to local packages.

    :param ctx: click context
    :param item_type: str type of item (connection/protocol/skill).
    :param item_id: the public id of the item.
    """
    item_type_plural = item_type + "s"
    try:
        # try non vendor first
        source_path = try_get_item_source_path(
            ctx.cwd, None, item_type_plural, item_id.name
        )
    except ClickException:
        # failed on user's packages
        #  try vendors
        source_path = try_get_item_source_path(
            os.path.join(ctx.cwd, "vendor"),
            item_id.author,
            item_type_plural,
            item_id.name,
        )

    check_package_public_id(source_path, item_type, item_id)

    try:
        registry_path = ctx.registry_path
    except ValueError as e:  # pragma: nocover
        raise click.ClickException(str(e))
    target_path = try_get_item_target_path(
        registry_path,
        item_id.author,
        item_type_plural,
        item_id.name,
    )
    copytree(source_path, target_path)
    click.echo(
        f'{item_type.title()} "{item_id}" successfully saved in packages folder.'
    )


def push_item_from_path(
    context: click.Context,
    component_type: str,
    path: Path,
    registry: str,
    package_type_config_class: Optional[Dict] = None,
) -> None:
    """Push item from path."""

    ctx = cast(Context, context.obj)
    component_path = Path(path)
    item_config = load_item_config(
        component_type,
        component_path,
        package_type_config_class=package_type_config_class,
    )

    if registry == REGISTRY_LOCAL:
        push_item_local(ctx, component_type, component_path, item_config.public_id)
    else:
        if get_default_remote_registry() == REMOTE_HTTP:
            raise click.ClickException(
                "Pushing using HTTP is not supported using path parameter."
            )

        push_item_ipfs(component_path, item_config.public_id)


def push_item_local(
    ctx: Context, item_type: str, component_path: Path, item_id: PublicId
) -> None:
    """Push items to the local registry."""
    registry_path: Optional[str]
    item_type_plural = item_type + "s"

    try:
        registry_path = ctx.registry_path
    except ValueError:  # pragma: nocover
        registry_path = get_registry_path_from_cli_config()
        if registry_path is None:
            raise click.ClickException("Registry path was not provided.")
        registry_path = cast(str, registry_path)

    target_path = try_get_item_target_path(
        registry_path,
        item_id.author,
        item_type_plural,
        item_id.name,
    )
    copytree(component_path, target_path)
    click.echo(
        f'{item_type.title()} "{item_id}" successfully saved in packages folder.'
    )


def push_item_ipfs(component_path: Path, public_id: PublicId, retries: int = 1) -> None:
    """Push items to the ipfs registry."""
    if not IS_IPFS_PLUGIN_INSTALLED:
        raise click.ClickException(
            "Please install ipfs plugin using `pip3 install open-aea-cli-ipfs`"
        )

    if len(list(component_path.glob("**/__pycache__"))) > 0:
        raise click.ClickException(
            f"Please remove all cache files from {component_path}"
        )

    ipfs_tool = IPFSTool(get_ipfs_node_multiaddr())
    for try_ in range(1, retries + 1):
        try:
            _, package_hash, _ = ipfs_tool.add(str(component_path))
            package_hash = to_v1(package_hash)
            break
        except Exception as e:  # pylint: disable=broad-except
            if try_ == retries:
                raise
            click.echo(f"Error occured: {repr(e)}. Trying one more time")

    click.echo("Pushed component with:")
    click.echo(f"\tPublicId: {public_id}")
    click.echo(f"\tPackage hash: {package_hash}")



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Check amount of time and mem for agent setup."""

from typing import Any, List, Tuple

import click
from aea_cli_benchmark.utils import (
    multi_run,
    number_of_runs_deco,
    output_format_deco,
    print_results,
)


@click.command(name="agent_construction_time")
@click.option(
    "--agents", default=25, help="Amount of agents to construct.", show_default=True
)
@number_of_runs_deco
@output_format_deco
def main(agents: int, number_of_runs: int, output_format: str) -> Any:
    """Check agents construction time and memory usage."""
    from aea_cli_benchmark.case_agent_construction_time.case import run

    parameters = {"Agents": agents, "Number of runs": number_of_runs}

    def result_fn() -> List[Tuple[str, Any, Any, Any]]:
        return multi_run(
            int(number_of_runs),
            run,
            (agents,),
        )

    return print_results(output_format, parameters, result_fn)



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of the 'aea fetch' subcommand."""
import os
import shutil
import tempfile
from distutils.dir_util import copy_tree  # pylint: disable=deprecated-module
from pathlib import Path
from typing import Optional, Union, cast

import click

from aea.cli.add import add_item
from aea.cli.registry.fetch import fetch_agent
from aea.cli.registry.settings import REGISTRY_LOCAL, REGISTRY_REMOTE, REMOTE_IPFS
from aea.cli.utils.click_utils import PublicIdParameter, registry_flag
from aea.cli.utils.config import (
    get_default_remote_registry,
    get_ipfs_node_multiaddr,
    try_to_load_agent_config,
)
from aea.cli.utils.context import Context
from aea.cli.utils.decorators import clean_after
from aea.cli.utils.loggers import logger
from aea.cli.utils.package_utils import try_get_item_source_path
from aea.configurations.base import PublicId
from aea.configurations.constants import (
    AGENTS,
    CONNECTION,
    CONTRACT,
    DEFAULT_AEA_CONFIG_FILE,
    PROTOCOL,
    SKILL,
)
from aea.exceptions import enforce
from aea.helpers.io import open_file


try:
    from aea_cli_ipfs.ipfs_utils import DownloadError, IPFSTool  # type: ignore

    IS_IPFS_PLUGIN_INSTALLED = True
except ImportError:
    IS_IPFS_PLUGIN_INSTALLED = False


@click.command(name="fetch")
@registry_flag()
@click.option(
    "--alias",
    type=str,
    required=False,
    help="Provide a local alias for the agent.",
)
@click.argument("public-id", type=PublicIdParameter(), required=True)
@click.pass_context
def fetch(
    click_context: click.Context, public_id: PublicId, alias: str, registry: str
) -> None:
    """Fetch an agent from the registry."""
    ctx = cast(Context, click_context.obj)
    ctx.registry_type = registry
    try:
        do_fetch(ctx, public_id, alias)
    except NotAnAgentPackage as e:
        raise click.ClickException(str(e)) from e


def do_fetch(
    ctx: Context,
    public_id: PublicId,
    alias: Optional[str] = None,
    target_dir: Optional[str] = None,
) -> None:
    """
    Run the Fetch command.

    :param ctx: the CLI context.
    :param public_id: the public id.
    :param alias: the agent alias.
    :param target_dir: the target directory, in case fetching locally.
    """
    if ctx.registry_type == REGISTRY_LOCAL:
        ctx.set_config("is_local", True)
        fetch_agent_locally(ctx, public_id, alias=alias, target_dir=target_dir)
    elif ctx.registry_type == REGISTRY_REMOTE:
        fetch_agent_remote(ctx, public_id, alias=alias, target_dir=target_dir)
    else:
        fetch_mixed(ctx, public_id, alias=alias, target_dir=target_dir)


def fetch_agent_remote(
    ctx: Context,
    public_id: PublicId,
    alias: Optional[str] = None,
    target_dir: Optional[str] = None,
) -> None:
    """
    Fetch Agent from remote registry.

    :param ctx: Context
    :param public_id: str public ID of desirable agent.
    :param alias: an optional alias.
    :param target_dir: the target directory to which the agent is fetched.
    """

    if get_default_remote_registry() == REMOTE_IPFS:
        fetch_agent_ipfs(ctx, public_id, alias, target_dir)
    else:
        # http

        fetch_agent(ctx, public_id, alias=alias, target_dir=target_dir)


def _is_version_correct(ctx: Context, agent_public_id: PublicId) -> bool:
    """
    Compare agent version to the one in public ID.

    :param ctx: Context object.
    :param agent_public_id: public ID of an agent.

    :return: bool is version correct.
    """
    return ctx.agent_config.public_id.same_prefix(agent_public_id) and (
        agent_public_id.package_version.is_latest
        or ctx.agent_config.version == agent_public_id.version
    )


@clean_after
def fetch_agent_ipfs(
    ctx: Context,
    public_id: PublicId,
    alias: Optional[str] = None,
    target_dir: Optional[Union[str, Path]] = None,
) -> None:
    """Fetch agent from ipfs."""

    if not IS_IPFS_PLUGIN_INSTALLED:
        raise click.ClickException(
            "Please install ipfs plugin using `pip3 install open-aea-cli-ipfs`"
        )

    if target_dir is None:
        target_dir = Path(ctx.cwd).absolute()
        target_dir = target_dir / (alias or public_id.name)

    target_dir = cast(Path, target_dir)
    if target_dir.exists():
        raise click.ClickException(f"Package already exists at {target_dir}")

    ipfs_tool = IPFSTool(get_ipfs_node_multiaddr())

    try:
        with tempfile.TemporaryDirectory() as temp_dir:
            agent_path = ipfs_tool.download(public_id.hash, str(temp_dir))
            shutil.move(agent_path, target_dir)
    except DownloadError as e:
        raise click.ClickException(
            f"Error occured while downloading agent {public_id}"
        ) from e
    except shutil.Error as e:
        raise click.ClickException(str(e)) from e

    ctx.cwd = str(target_dir)
    if not Path(target_dir, DEFAULT_AEA_CONFIG_FILE).exists():
        raise NotAnAgentPackage(
            f"Downloaded packages at {target_dir} is not an agent package, please check hash"
        )

    try_to_load_agent_config(ctx)

    ctx.agent_config.agent_name = target_dir.name
    ctx.agent_loader.dump(
        ctx.agent_config,
        open_file(os.path.join(ctx.cwd, DEFAULT_AEA_CONFIG_FILE), "w"),
    )

    try_to_load_agent_config(ctx)
    _fetch_agent_deps(ctx)
    click.echo("Agent {} successfully fetched.".format(ctx.agent_config.agent_name))


@clean_after
def fetch_agent_locally(
    ctx: Context,
    public_id: PublicId,
    alias: Optional[str] = None,
    target_dir: Optional[str] = None,
) -> None:
    """
    Fetch Agent from local packages.

    :param ctx: a Context object.
    :param public_id: public ID of agent to be fetched.
    :param alias: an optional alias.
    :param target_dir: the target directory to which the agent is fetched.
    """
    try:
        registry_path = ctx.registry_path
    except ValueError as e:
        raise click.ClickException(str(e))

    source_path = try_get_item_source_path(
        registry_path, public_id.author, AGENTS, public_id.name
    )
    enforce(
        ctx.config.get("is_local") is True or ctx.config.get("is_mixed") is True,
        "Please use `ctx.set_config('is_local', True)` or `ctx.set_config('is_mixed', True)` to fetch agent and all components locally.",
    )
    try_to_load_agent_config(ctx, agent_src_path=source_path)
    if not _is_version_correct(ctx, public_id):
        raise click.ClickException(
            "Wrong agent version in public ID: specified {}, found {}.".format(
                public_id.version, ctx.agent_config.version
            )
        )

    folder_name = target_dir or (public_id.name if alias is None else alias)
    target_path = os.path.join(ctx.cwd, folder_name)
    if os.path.exists(target_path):
        path = Path(target_path)
        raise click.ClickException(
            f'Item "{path.name}" already exists in target folder "{path.parent}".'
        )
    if target_dir is not None:
        os.makedirs(target_path)  # pragma: nocover

    ctx.clean_paths.append(target_path)
    copy_tree(source_path, target_path)

    ctx.cwd = target_path
    try_to_load_agent_config(ctx)

    if alias is not None:
        ctx.agent_config.agent_name = alias
        ctx.agent_loader.dump(
            ctx.agent_config,
            open_file(os.path.join(ctx.cwd, DEFAULT_AEA_CONFIG_FILE), "w"),
        )

    _fetch_agent_deps(ctx)
    click.echo("Agent {} successfully fetched.".format(public_id.name))


def _fetch_agent_deps(ctx: Context) -> None:
    """
    Fetch agent dependencies.

    :param ctx: context object.
    """
    for item_type in (PROTOCOL, CONTRACT, CONNECTION, SKILL):
        item_type_plural = "{}s".format(item_type)
        required_items = cast(set, getattr(ctx.agent_config, item_type_plural))
        required_items_check = required_items.copy()
        try:
            for item_id in required_items:
                add_item(ctx, item_type, item_id)
        except RuntimeError as e:
            missing_deps = required_items_check.symmetric_difference(required_items)
            error = "\n- ".join(
                [
                    "Size of the dependency set changed during the iteration; "
                    "Following dependencies are missing from the agent configuration: ",
                    *map(lambda x: str(x.without_hash()), missing_deps),
                ]
            )
            raise click.ClickException(error) from e


def fetch_mixed(
    ctx: Context,
    public_id: PublicId,
    alias: Optional[str] = None,
    target_dir: Optional[str] = None,
) -> None:
    """
    Fetch an agent in mixed mode.

    :param ctx: the Context.
    :param public_id: the public id.
    :param alias: the alias to the agent.
    :param target_dir: the target directory.
    """
    try:
        fetch_agent_locally(ctx, public_id, alias=alias, target_dir=target_dir)
    except click.ClickException as e:
        logger.debug(
            f"Fetch from local registry failed (reason={str(e)}), trying remote registry..."
        )
        fetch_agent_remote(ctx, public_id, alias=alias, target_dir=target_dir)


class NotAnAgentPackage(Exception):
    """Raise when downloaded package is not an agent package."""



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2023 Valory AG
#   Copyright 2018-2020 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Implementation of the 'aea generate_key' subcommand."""
import json
from pathlib import Path
from typing import Dict, Optional, Union

import click

from aea.cli.add_key import _add_private_key
from aea.cli.utils.click_utils import LedgerChoice, password_option
from aea.cli.utils.decorators import _check_aea_project
from aea.configurations.constants import (
    ADDRESS,
    MULTIKEY_FILENAME,
    PRIVATE_KEY,
    PRIVATE_KEY_PATH_SCHEMA,
)
from aea.crypto.helpers import create_private_key
from aea.crypto.registries import crypto_registry, make_crypto


@click.command()
@click.argument(
    "type_",
    metavar="TYPE",
    type=LedgerChoice(),
    required=True,
)
@click.argument(
    "file",
    metavar="FILE",
    type=click.Path(exists=False, file_okay=True, dir_okay=False, readable=True),
    required=False,
)
@password_option(confirmation_prompt=True)
@click.option(
    "--add-key",
    is_flag=True,
    help="Add generated key.",
)
@click.option(
    "--connection", is_flag=True, help="For adding a private key for connections."
)
@click.option(
    "--extra-entropy",
    type=str,
    required=False,
    default="",
)
@click.option("-n", type=int, help="Number of keys to generate")
@click.pass_context
def generate_key(
    click_context: click.core.Context,
    type_: str,
    file: str,
    password: Optional[str],
    add_key: bool = False,
    connection: bool = False,
    extra_entropy: Union[str, bytes, int] = "",
    n: Optional[int] = None,
) -> None:
    """Generate a private key and place it in a file."""
    if n is None:
        _generate_one(
            click_context=click_context,
            type_=type_,
            file=file,
            password=password,
            add_key=add_key,
            connection=connection,
            extra_entropy=extra_entropy,
        )
        return

    _generate_multiple_keys(
        n=n,
        type_=type_,
        password=password,
        extra_entropy=extra_entropy,
        file=file,
    )


def _generate_one(
    click_context: click.core.Context,
    type_: str,
    file: str,
    password: Optional[str],
    add_key: bool = False,
    connection: bool = False,
    extra_entropy: Union[str, bytes, int] = "",
) -> None:
    """Generate one key."""
    keys_generated = _generate_private_key(type_, file, password, extra_entropy)
    if add_key:
        _check_aea_project((click_context,))
        for key_type, key_filename in keys_generated.items():
            _add_private_key(
                click_context, key_type, key_filename, password, connection
            )


def _generate_private_key(
    type_: str,
    file: Optional[str] = None,
    password: Optional[str] = None,
    extra_entropy: Union[str, bytes, int] = "",
) -> Dict[str, str]:
    """
    Generate private key.

    :param type_: type.
    :param file: path to file.
    :param password: the password to encrypt/decrypt the private key.
    :param extra_entropy: add extra randomness to whatever randomness your OS can provide

    :return: dict of types and filenames of keys generated
    """
    keys = {}
    if type_ == "all" and file is not None:
        raise click.ClickException("Type all cannot be used in combination with file.")
    types = list(crypto_registry.supported_ids) if type_ == "all" else [type_]
    for type__ in types:
        private_key_file = (
            PRIVATE_KEY_PATH_SCHEMA.format(type__) if file is None else file
        )
        if _can_write(private_key_file):
            create_private_key(type__, private_key_file, password, extra_entropy)
        keys[type__] = private_key_file
    return keys


def _generate_multiple_keys(
    n: int,
    type_: str,
    password: Optional[str] = None,
    extra_entropy: Union[str, bytes, int] = "",
    file: Optional[str] = None,
) -> None:
    """Generate n key pairs."""

    key_pairs = []
    for _ in range(n):
        crypto = make_crypto(type_, extra_entropy=extra_entropy)
        priv_key = (
            crypto.encrypt(password=password)
            if password is not None
            else crypto.private_key
        )
        key_pairs.append({ADDRESS: crypto.address, PRIVATE_KEY: priv_key})

    file = file or MULTIKEY_FILENAME
    if _can_write(file):
        Path(file).write_text(json.dumps(obj=key_pairs, indent=2), encoding="utf-8")


def _can_write(path: str) -> bool:
    if Path(path).exists():
        value = click.confirm(
            "The file {} already exists. Do you want to overwrite it?".format(path),
            default=False,
        )
        return value
    return True



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Envelopes generation speed for Behaviour act test."""
import time
from threading import Thread
from typing import List, Tuple, Union, cast

from aea_cli_benchmark.utils import SyncedGeneratorConnection  # noqa: I100
from aea_cli_benchmark.utils import (
    make_agent,
    make_envelope,
    make_skill,
    wait_for_condition,
)

from aea.registries.resources import Resources
from aea.skills.base import Behaviour

from packages.fetchai.protocols.default.message import DefaultMessage


class TestBehaviour(Behaviour):
    """Dummy handler to handle messages."""

    _tick_interval = 1

    SUPPORTED_PROTOCOL = DefaultMessage.protocol_id

    def setup(self) -> None:
        """Set up behaviour."""
        self.count = 0  # pylint: disable=attribute-defined-outside-init

    def teardown(self) -> None:
        """Tear up behaviour."""

    def act(self) -> None:
        """Perform action on periodic basis."""
        s = time.time()
        while time.time() - s < self.tick_interval:
            self.context.outbox.put(make_envelope("1", "2"))
            self.count += 1


def run(duration: int, runtime_mode: str) -> List[Tuple[str, Union[int, float]]]:
    """Test act message generate performance."""
    # pylint: disable=import-outside-toplevel,unused-import
    # import manually due to some lazy imports in decision_maker
    import aea.decision_maker.default  # noqa: F401

    resources = Resources()
    connection = SyncedGeneratorConnection.make()
    resources.add_connection(connection)
    agent = make_agent(runtime_mode=runtime_mode, resources=resources)
    skill = make_skill(agent, behaviours={"test": TestBehaviour})
    agent.resources.add_skill(skill)
    t = Thread(target=agent.start, daemon=True)
    t.start()
    wait_for_condition(lambda: agent.is_running, timeout=5)
    time.sleep(duration)
    agent.stop()
    t.join(5)

    rate = connection.count_in / duration
    return [
        ("envelopes sent", cast(TestBehaviour, skill.behaviours["test"]).count),
        ("envelopes received", connection.count_in),
        ("rate(envelopes/second)", rate),
    ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2023 fetchai
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains tac's message definition."""

# pylint: disable=too-many-statements,too-many-locals,no-member,too-few-public-methods,too-many-branches,not-an-iterable,unidiomatic-typecheck,unsubscriptable-object
import logging
from typing import Any, Dict, Optional, Set, Tuple, cast

from aea.configurations.base import PublicId
from aea.exceptions import AEAEnforceError, enforce
from aea.protocols.base import Message  # type: ignore

from packages.fetchai.protocols.tac.custom_types import ErrorCode as CustomErrorCode


_default_logger = logging.getLogger("aea.packages.fetchai.protocols.tac.message")

DEFAULT_BODY_SIZE = 4


class TacMessage(Message):
    """The tac protocol implements the messages an AEA needs to participate in the TAC."""

    protocol_id = PublicId.from_str("fetchai/tac:1.0.0")
    protocol_specification_id = PublicId.from_str("fetchai/tac:1.0.0")

    ErrorCode = CustomErrorCode

    class Performative(Message.Performative):
        """Performatives for the tac protocol."""

        CANCELLED = "cancelled"
        GAME_DATA = "game_data"
        REGISTER = "register"
        TAC_ERROR = "tac_error"
        TRANSACTION = "transaction"
        TRANSACTION_CONFIRMATION = "transaction_confirmation"
        UNREGISTER = "unregister"

        def __str__(self) -> str:
            """Get the string representation."""
            return str(self.value)

    _performatives = {
        "cancelled",
        "game_data",
        "register",
        "tac_error",
        "transaction",
        "transaction_confirmation",
        "unregister",
    }
    __slots__: Tuple[str, ...] = tuple()

    class _SlotsCls:
        __slots__ = (
            "agent_addr_to_name",
            "agent_name",
            "amount_by_currency_id",
            "counterparty_address",
            "counterparty_signature",
            "currency_id_to_name",
            "dialogue_reference",
            "error_code",
            "exchange_params_by_currency_id",
            "fee_by_currency_id",
            "good_id_to_name",
            "info",
            "ledger_id",
            "message_id",
            "nonce",
            "performative",
            "quantities_by_good_id",
            "sender_address",
            "sender_signature",
            "target",
            "transaction_id",
            "utility_params_by_good_id",
            "version_id",
        )

    def __init__(
        self,
        performative: Performative,
        dialogue_reference: Tuple[str, str] = ("", ""),
        message_id: int = 1,
        target: int = 0,
        **kwargs: Any,
    ):
        """
        Initialise an instance of TacMessage.

        :param message_id: the message id.
        :param dialogue_reference: the dialogue reference.
        :param target: the message target.
        :param performative: the message performative.
        :param **kwargs: extra options.
        """
        super().__init__(
            dialogue_reference=dialogue_reference,
            message_id=message_id,
            target=target,
            performative=TacMessage.Performative(performative),
            **kwargs,
        )

    @property
    def valid_performatives(self) -> Set[str]:
        """Get valid performatives."""
        return self._performatives

    @property
    def dialogue_reference(self) -> Tuple[str, str]:
        """Get the dialogue_reference of the message."""
        enforce(self.is_set("dialogue_reference"), "dialogue_reference is not set.")
        return cast(Tuple[str, str], self.get("dialogue_reference"))

    @property
    def message_id(self) -> int:
        """Get the message_id of the message."""
        enforce(self.is_set("message_id"), "message_id is not set.")
        return cast(int, self.get("message_id"))

    @property
    def performative(self) -> Performative:  # type: ignore # noqa: F821
        """Get the performative of the message."""
        enforce(self.is_set("performative"), "performative is not set.")
        return cast(TacMessage.Performative, self.get("performative"))

    @property
    def target(self) -> int:
        """Get the target of the message."""
        enforce(self.is_set("target"), "target is not set.")
        return cast(int, self.get("target"))

    @property
    def agent_addr_to_name(self) -> Dict[str, str]:
        """Get the 'agent_addr_to_name' content from the message."""
        enforce(
            self.is_set("agent_addr_to_name"),
            "'agent_addr_to_name' content is not set.",
        )
        return cast(Dict[str, str], self.get("agent_addr_to_name"))

    @property
    def agent_name(self) -> str:
        """Get the 'agent_name' content from the message."""
        enforce(self.is_set("agent_name"), "'agent_name' content is not set.")
        return cast(str, self.get("agent_name"))

    @property
    def amount_by_currency_id(self) -> Dict[str, int]:
        """Get the 'amount_by_currency_id' content from the message."""
        enforce(
            self.is_set("amount_by_currency_id"),
            "'amount_by_currency_id' content is not set.",
        )
        return cast(Dict[str, int], self.get("amount_by_currency_id"))

    @property
    def counterparty_address(self) -> str:
        """Get the 'counterparty_address' content from the message."""
        enforce(
            self.is_set("counterparty_address"),
            "'counterparty_address' content is not set.",
        )
        return cast(str, self.get("counterparty_address"))

    @property
    def counterparty_signature(self) -> str:
        """Get the 'counterparty_signature' content from the message."""
        enforce(
            self.is_set("counterparty_signature"),
            "'counterparty_signature' content is not set.",
        )
        return cast(str, self.get("counterparty_signature"))

    @property
    def currency_id_to_name(self) -> Dict[str, str]:
        """Get the 'currency_id_to_name' content from the message."""
        enforce(
            self.is_set("currency_id_to_name"),
            "'currency_id_to_name' content is not set.",
        )
        return cast(Dict[str, str], self.get("currency_id_to_name"))

    @property
    def error_code(self) -> CustomErrorCode:
        """Get the 'error_code' content from the message."""
        enforce(self.is_set("error_code"), "'error_code' content is not set.")
        return cast(CustomErrorCode, self.get("error_code"))

    @property
    def exchange_params_by_currency_id(self) -> Dict[str, float]:
        """Get the 'exchange_params_by_currency_id' content from the message."""
        enforce(
            self.is_set("exchange_params_by_currency_id"),
            "'exchange_params_by_currency_id' content is not set.",
        )
        return cast(Dict[str, float], self.get("exchange_params_by_currency_id"))

    @property
    def fee_by_currency_id(self) -> Dict[str, int]:
        """Get the 'fee_by_currency_id' content from the message."""
        enforce(
            self.is_set("fee_by_currency_id"),
            "'fee_by_currency_id' content is not set.",
        )
        return cast(Dict[str, int], self.get("fee_by_currency_id"))

    @property
    def good_id_to_name(self) -> Dict[str, str]:
        """Get the 'good_id_to_name' content from the message."""
        enforce(self.is_set("good_id_to_name"), "'good_id_to_name' content is not set.")
        return cast(Dict[str, str], self.get("good_id_to_name"))

    @property
    def info(self) -> Optional[Dict[str, str]]:
        """Get the 'info' content from the message."""
        return cast(Optional[Dict[str, str]], self.get("info"))

    @property
    def ledger_id(self) -> str:
        """Get the 'ledger_id' content from the message."""
        enforce(self.is_set("ledger_id"), "'ledger_id' content is not set.")
        return cast(str, self.get("ledger_id"))

    @property
    def nonce(self) -> str:
        """Get the 'nonce' content from the message."""
        enforce(self.is_set("nonce"), "'nonce' content is not set.")
        return cast(str, self.get("nonce"))

    @property
    def quantities_by_good_id(self) -> Dict[str, int]:
        """Get the 'quantities_by_good_id' content from the message."""
        enforce(
            self.is_set("quantities_by_good_id"),
            "'quantities_by_good_id' content is not set.",
        )
        return cast(Dict[str, int], self.get("quantities_by_good_id"))

    @property
    def sender_address(self) -> str:
        """Get the 'sender_address' content from the message."""
        enforce(self.is_set("sender_address"), "'sender_address' content is not set.")
        return cast(str, self.get("sender_address"))

    @property
    def sender_signature(self) -> str:
        """Get the 'sender_signature' content from the message."""
        enforce(
            self.is_set("sender_signature"), "'sender_signature' content is not set."
        )
        return cast(str, self.get("sender_signature"))

    @property
    def transaction_id(self) -> str:
        """Get the 'transaction_id' content from the message."""
        enforce(self.is_set("transaction_id"), "'transaction_id' content is not set.")
        return cast(str, self.get("transaction_id"))

    @property
    def utility_params_by_good_id(self) -> Dict[str, float]:
        """Get the 'utility_params_by_good_id' content from the message."""
        enforce(
            self.is_set("utility_params_by_good_id"),
            "'utility_params_by_good_id' content is not set.",
        )
        return cast(Dict[str, float], self.get("utility_params_by_good_id"))

    @property
    def version_id(self) -> str:
        """Get the 'version_id' content from the message."""
        enforce(self.is_set("version_id"), "'version_id' content is not set.")
        return cast(str, self.get("version_id"))

    def _is_consistent(self) -> bool:
        """Check that the message follows the tac protocol."""
        try:
            enforce(
                isinstance(self.dialogue_reference, tuple),
                "Invalid type for 'dialogue_reference'. Expected 'tuple'. Found '{}'.".format(
                    type(self.dialogue_reference)
                ),
            )
            enforce(
                isinstance(self.dialogue_reference[0], str),
                "Invalid type for 'dialogue_reference[0]'. Expected 'str'. Found '{}'.".format(
                    type(self.dialogue_reference[0])
                ),
            )
            enforce(
                isinstance(self.dialogue_reference[1], str),
                "Invalid type for 'dialogue_reference[1]'. Expected 'str'. Found '{}'.".format(
                    type(self.dialogue_reference[1])
                ),
            )
            enforce(
                type(self.message_id) is int,
                "Invalid type for 'message_id'. Expected 'int'. Found '{}'.".format(
                    type(self.message_id)
                ),
            )
            enforce(
                type(self.target) is int,
                "Invalid type for 'target'. Expected 'int'. Found '{}'.".format(
                    type(self.target)
                ),
            )

            # Light Protocol Rule 2
            # Check correct performative
            enforce(
                isinstance(self.performative, TacMessage.Performative),
                "Invalid 'performative'. Expected either of '{}'. Found '{}'.".format(
                    self.valid_performatives, self.performative
                ),
            )

            # Check correct contents
            actual_nb_of_contents = len(self._body) - DEFAULT_BODY_SIZE
            expected_nb_of_contents = 0
            if self.performative == TacMessage.Performative.REGISTER:
                expected_nb_of_contents = 1
                enforce(
                    isinstance(self.agent_name, str),
                    "Invalid type for content 'agent_name'. Expected 'str'. Found '{}'.".format(
                        type(self.agent_name)
                    ),
                )
            elif self.performative == TacMessage.Performative.UNREGISTER:
                expected_nb_of_contents = 0
            elif self.performative == TacMessage.Performative.TRANSACTION:
                expected_nb_of_contents = 10
                enforce(
                    isinstance(self.transaction_id, str),
                    "Invalid type for content 'transaction_id'. Expected 'str'. Found '{}'.".format(
                        type(self.transaction_id)
                    ),
                )
                enforce(
                    isinstance(self.ledger_id, str),
                    "Invalid type for content 'ledger_id'. Expected 'str'. Found '{}'.".format(
                        type(self.ledger_id)
                    ),
                )
                enforce(
                    isinstance(self.sender_address, str),
                    "Invalid type for content 'sender_address'. Expected 'str'. Found '{}'.".format(
                        type(self.sender_address)
                    ),
                )
                enforce(
                    isinstance(self.counterparty_address, str),
                    "Invalid type for content 'counterparty_address'. Expected 'str'. Found '{}'.".format(
                        type(self.counterparty_address)
                    ),
                )
                enforce(
                    isinstance(self.amount_by_currency_id, dict),
                    "Invalid type for content 'amount_by_currency_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.amount_by_currency_id)
                    ),
                )
                for (
                    key_of_amount_by_currency_id,
                    value_of_amount_by_currency_id,
                ) in self.amount_by_currency_id.items():
                    enforce(
                        isinstance(key_of_amount_by_currency_id, str),
                        "Invalid type for dictionary keys in content 'amount_by_currency_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_amount_by_currency_id)
                        ),
                    )
                    enforce(
                        type(value_of_amount_by_currency_id) is int,
                        "Invalid type for dictionary values in content 'amount_by_currency_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_amount_by_currency_id)
                        ),
                    )
                enforce(
                    isinstance(self.fee_by_currency_id, dict),
                    "Invalid type for content 'fee_by_currency_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.fee_by_currency_id)
                    ),
                )
                for (
                    key_of_fee_by_currency_id,
                    value_of_fee_by_currency_id,
                ) in self.fee_by_currency_id.items():
                    enforce(
                        isinstance(key_of_fee_by_currency_id, str),
                        "Invalid type for dictionary keys in content 'fee_by_currency_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_fee_by_currency_id)
                        ),
                    )
                    enforce(
                        type(value_of_fee_by_currency_id) is int,
                        "Invalid type for dictionary values in content 'fee_by_currency_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_fee_by_currency_id)
                        ),
                    )
                enforce(
                    isinstance(self.quantities_by_good_id, dict),
                    "Invalid type for content 'quantities_by_good_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.quantities_by_good_id)
                    ),
                )
                for (
                    key_of_quantities_by_good_id,
                    value_of_quantities_by_good_id,
                ) in self.quantities_by_good_id.items():
                    enforce(
                        isinstance(key_of_quantities_by_good_id, str),
                        "Invalid type for dictionary keys in content 'quantities_by_good_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_quantities_by_good_id)
                        ),
                    )
                    enforce(
                        type(value_of_quantities_by_good_id) is int,
                        "Invalid type for dictionary values in content 'quantities_by_good_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_quantities_by_good_id)
                        ),
                    )
                enforce(
                    isinstance(self.nonce, str),
                    "Invalid type for content 'nonce'. Expected 'str'. Found '{}'.".format(
                        type(self.nonce)
                    ),
                )
                enforce(
                    isinstance(self.sender_signature, str),
                    "Invalid type for content 'sender_signature'. Expected 'str'. Found '{}'.".format(
                        type(self.sender_signature)
                    ),
                )
                enforce(
                    isinstance(self.counterparty_signature, str),
                    "Invalid type for content 'counterparty_signature'. Expected 'str'. Found '{}'.".format(
                        type(self.counterparty_signature)
                    ),
                )
            elif self.performative == TacMessage.Performative.CANCELLED:
                expected_nb_of_contents = 0
            elif self.performative == TacMessage.Performative.GAME_DATA:
                expected_nb_of_contents = 9
                enforce(
                    isinstance(self.amount_by_currency_id, dict),
                    "Invalid type for content 'amount_by_currency_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.amount_by_currency_id)
                    ),
                )
                for (
                    key_of_amount_by_currency_id,
                    value_of_amount_by_currency_id,
                ) in self.amount_by_currency_id.items():
                    enforce(
                        isinstance(key_of_amount_by_currency_id, str),
                        "Invalid type for dictionary keys in content 'amount_by_currency_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_amount_by_currency_id)
                        ),
                    )
                    enforce(
                        type(value_of_amount_by_currency_id) is int,
                        "Invalid type for dictionary values in content 'amount_by_currency_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_amount_by_currency_id)
                        ),
                    )
                enforce(
                    isinstance(self.exchange_params_by_currency_id, dict),
                    "Invalid type for content 'exchange_params_by_currency_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.exchange_params_by_currency_id)
                    ),
                )
                for (
                    key_of_exchange_params_by_currency_id,
                    value_of_exchange_params_by_currency_id,
                ) in self.exchange_params_by_currency_id.items():
                    enforce(
                        isinstance(key_of_exchange_params_by_currency_id, str),
                        "Invalid type for dictionary keys in content 'exchange_params_by_currency_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_exchange_params_by_currency_id)
                        ),
                    )
                    enforce(
                        isinstance(value_of_exchange_params_by_currency_id, float),
                        "Invalid type for dictionary values in content 'exchange_params_by_currency_id'. Expected 'float'. Found '{}'.".format(
                            type(value_of_exchange_params_by_currency_id)
                        ),
                    )
                enforce(
                    isinstance(self.quantities_by_good_id, dict),
                    "Invalid type for content 'quantities_by_good_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.quantities_by_good_id)
                    ),
                )
                for (
                    key_of_quantities_by_good_id,
                    value_of_quantities_by_good_id,
                ) in self.quantities_by_good_id.items():
                    enforce(
                        isinstance(key_of_quantities_by_good_id, str),
                        "Invalid type for dictionary keys in content 'quantities_by_good_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_quantities_by_good_id)
                        ),
                    )
                    enforce(
                        type(value_of_quantities_by_good_id) is int,
                        "Invalid type for dictionary values in content 'quantities_by_good_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_quantities_by_good_id)
                        ),
                    )
                enforce(
                    isinstance(self.utility_params_by_good_id, dict),
                    "Invalid type for content 'utility_params_by_good_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.utility_params_by_good_id)
                    ),
                )
                for (
                    key_of_utility_params_by_good_id,
                    value_of_utility_params_by_good_id,
                ) in self.utility_params_by_good_id.items():
                    enforce(
                        isinstance(key_of_utility_params_by_good_id, str),
                        "Invalid type for dictionary keys in content 'utility_params_by_good_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_utility_params_by_good_id)
                        ),
                    )
                    enforce(
                        isinstance(value_of_utility_params_by_good_id, float),
                        "Invalid type for dictionary values in content 'utility_params_by_good_id'. Expected 'float'. Found '{}'.".format(
                            type(value_of_utility_params_by_good_id)
                        ),
                    )
                enforce(
                    isinstance(self.fee_by_currency_id, dict),
                    "Invalid type for content 'fee_by_currency_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.fee_by_currency_id)
                    ),
                )
                for (
                    key_of_fee_by_currency_id,
                    value_of_fee_by_currency_id,
                ) in self.fee_by_currency_id.items():
                    enforce(
                        isinstance(key_of_fee_by_currency_id, str),
                        "Invalid type for dictionary keys in content 'fee_by_currency_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_fee_by_currency_id)
                        ),
                    )
                    enforce(
                        type(value_of_fee_by_currency_id) is int,
                        "Invalid type for dictionary values in content 'fee_by_currency_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_fee_by_currency_id)
                        ),
                    )
                enforce(
                    isinstance(self.agent_addr_to_name, dict),
                    "Invalid type for content 'agent_addr_to_name'. Expected 'dict'. Found '{}'.".format(
                        type(self.agent_addr_to_name)
                    ),
                )
                for (
                    key_of_agent_addr_to_name,
                    value_of_agent_addr_to_name,
                ) in self.agent_addr_to_name.items():
                    enforce(
                        isinstance(key_of_agent_addr_to_name, str),
                        "Invalid type for dictionary keys in content 'agent_addr_to_name'. Expected 'str'. Found '{}'.".format(
                            type(key_of_agent_addr_to_name)
                        ),
                    )
                    enforce(
                        isinstance(value_of_agent_addr_to_name, str),
                        "Invalid type for dictionary values in content 'agent_addr_to_name'. Expected 'str'. Found '{}'.".format(
                            type(value_of_agent_addr_to_name)
                        ),
                    )
                enforce(
                    isinstance(self.currency_id_to_name, dict),
                    "Invalid type for content 'currency_id_to_name'. Expected 'dict'. Found '{}'.".format(
                        type(self.currency_id_to_name)
                    ),
                )
                for (
                    key_of_currency_id_to_name,
                    value_of_currency_id_to_name,
                ) in self.currency_id_to_name.items():
                    enforce(
                        isinstance(key_of_currency_id_to_name, str),
                        "Invalid type for dictionary keys in content 'currency_id_to_name'. Expected 'str'. Found '{}'.".format(
                            type(key_of_currency_id_to_name)
                        ),
                    )
                    enforce(
                        isinstance(value_of_currency_id_to_name, str),
                        "Invalid type for dictionary values in content 'currency_id_to_name'. Expected 'str'. Found '{}'.".format(
                            type(value_of_currency_id_to_name)
                        ),
                    )
                enforce(
                    isinstance(self.good_id_to_name, dict),
                    "Invalid type for content 'good_id_to_name'. Expected 'dict'. Found '{}'.".format(
                        type(self.good_id_to_name)
                    ),
                )
                for (
                    key_of_good_id_to_name,
                    value_of_good_id_to_name,
                ) in self.good_id_to_name.items():
                    enforce(
                        isinstance(key_of_good_id_to_name, str),
                        "Invalid type for dictionary keys in content 'good_id_to_name'. Expected 'str'. Found '{}'.".format(
                            type(key_of_good_id_to_name)
                        ),
                    )
                    enforce(
                        isinstance(value_of_good_id_to_name, str),
                        "Invalid type for dictionary values in content 'good_id_to_name'. Expected 'str'. Found '{}'.".format(
                            type(value_of_good_id_to_name)
                        ),
                    )
                enforce(
                    isinstance(self.version_id, str),
                    "Invalid type for content 'version_id'. Expected 'str'. Found '{}'.".format(
                        type(self.version_id)
                    ),
                )
                if self.is_set("info"):
                    expected_nb_of_contents += 1
                    info = cast(Dict[str, str], self.info)
                    enforce(
                        isinstance(info, dict),
                        "Invalid type for content 'info'. Expected 'dict'. Found '{}'.".format(
                            type(info)
                        ),
                    )
                    for key_of_info, value_of_info in info.items():
                        enforce(
                            isinstance(key_of_info, str),
                            "Invalid type for dictionary keys in content 'info'. Expected 'str'. Found '{}'.".format(
                                type(key_of_info)
                            ),
                        )
                        enforce(
                            isinstance(value_of_info, str),
                            "Invalid type for dictionary values in content 'info'. Expected 'str'. Found '{}'.".format(
                                type(value_of_info)
                            ),
                        )
            elif self.performative == TacMessage.Performative.TRANSACTION_CONFIRMATION:
                expected_nb_of_contents = 3
                enforce(
                    isinstance(self.transaction_id, str),
                    "Invalid type for content 'transaction_id'. Expected 'str'. Found '{}'.".format(
                        type(self.transaction_id)
                    ),
                )
                enforce(
                    isinstance(self.amount_by_currency_id, dict),
                    "Invalid type for content 'amount_by_currency_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.amount_by_currency_id)
                    ),
                )
                for (
                    key_of_amount_by_currency_id,
                    value_of_amount_by_currency_id,
                ) in self.amount_by_currency_id.items():
                    enforce(
                        isinstance(key_of_amount_by_currency_id, str),
                        "Invalid type for dictionary keys in content 'amount_by_currency_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_amount_by_currency_id)
                        ),
                    )
                    enforce(
                        type(value_of_amount_by_currency_id) is int,
                        "Invalid type for dictionary values in content 'amount_by_currency_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_amount_by_currency_id)
                        ),
                    )
                enforce(
                    isinstance(self.quantities_by_good_id, dict),
                    "Invalid type for content 'quantities_by_good_id'. Expected 'dict'. Found '{}'.".format(
                        type(self.quantities_by_good_id)
                    ),
                )
                for (
                    key_of_quantities_by_good_id,
                    value_of_quantities_by_good_id,
                ) in self.quantities_by_good_id.items():
                    enforce(
                        isinstance(key_of_quantities_by_good_id, str),
                        "Invalid type for dictionary keys in content 'quantities_by_good_id'. Expected 'str'. Found '{}'.".format(
                            type(key_of_quantities_by_good_id)
                        ),
                    )
                    enforce(
                        type(value_of_quantities_by_good_id) is int,
                        "Invalid type for dictionary values in content 'quantities_by_good_id'. Expected 'int'. Found '{}'.".format(
                            type(value_of_quantities_by_good_id)
                        ),
                    )
            elif self.performative == TacMessage.Performative.TAC_ERROR:
                expected_nb_of_contents = 1
                enforce(
                    isinstance(self.error_code, CustomErrorCode),
                    "Invalid type for content 'error_code'. Expected 'ErrorCode'. Found '{}'.".format(
                        type(self.error_code)
                    ),
                )
                if self.is_set("info"):
                    expected_nb_of_contents += 1
                    info = cast(Dict[str, str], self.info)
                    enforce(
                        isinstance(info, dict),
                        "Invalid type for content 'info'. Expected 'dict'. Found '{}'.".format(
                            type(info)
                        ),
                    )
                    for key_of_info, value_of_info in info.items():
                        enforce(
                            isinstance(key_of_info, str),
                            "Invalid type for dictionary keys in content 'info'. Expected 'str'. Found '{}'.".format(
                                type(key_of_info)
                            ),
                        )
                        enforce(
                            isinstance(value_of_info, str),
                            "Invalid type for dictionary values in content 'info'. Expected 'str'. Found '{}'.".format(
                                type(value_of_info)
                            ),
                        )

            # Check correct content count
            enforce(
                expected_nb_of_contents == actual_nb_of_contents,
                "Incorrect number of contents. Expected {}. Found {}".format(
                    expected_nb_of_contents, actual_nb_of_contents
                ),
            )

            # Light Protocol Rule 3
            if self.message_id == 1:
                enforce(
                    self.target == 0,
                    "Invalid 'target'. Expected 0 (because 'message_id' is 1). Found {}.".format(
                        self.target
                    ),
                )
        except (AEAEnforceError, ValueError, KeyError) as e:
            _default_logger.error(str(e))
            return False

        return True



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2021-2022 Valory AG
#   Copyright 2018-2019 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""This module contains tests for transaction."""
# pylint: skip-file

from typing import List, Type

from aea_ledger_cosmos import CosmosCrypto

from aea.helpers.transaction.base import (
    RawMessage,
    RawTransaction,
    SignedMessage,
    SignedTransaction,
    Terms,
)
from aea.protocols.base import Message
from aea.protocols.dialogue.base import Dialogue as BaseDialogue
from aea.protocols.dialogue.base import Dialogues
from aea.test_tools.test_protocol import (
    BaseProtocolDialoguesTestCase,
    BaseProtocolMessagesTestCase,
)

from packages.open_aea.protocols.signing.dialogues import (
    SigningDialogue as BaseSigningDialogue,
)
from packages.open_aea.protocols.signing.dialogues import (
    SigningDialogues as BaseSigningDialogues,
)
from packages.open_aea.protocols.signing.message import SigningMessage


class TestMessages(BaseProtocolMessagesTestCase):
    """Base class to test message construction for the protocol."""

    MESSAGE_CLASS = SigningMessage

    ledger_id = CosmosCrypto.identifier
    terms = Terms(
        ledger_id=ledger_id,
        sender_address="address1",
        counterparty_address="address2",
        amount_by_currency_id={"FET": -2},
        quantities_by_good_id={"good_id": 10},
        is_sender_payable_tx_fee=True,
        nonce="transaction nonce",
    )

    def build_messages(self) -> List[SigningMessage]:  # type: ignore[override]
        """Build the messages to be used for testing."""
        return [
            SigningMessage(
                performative=SigningMessage.Performative.SIGN_TRANSACTION,
                terms=self.terms,
                raw_transaction=RawTransaction(self.ledger_id, {"tx": "transaction"}),
            ),
            SigningMessage(
                performative=SigningMessage.Performative.SIGN_MESSAGE,
                terms=self.terms,
                raw_message=RawMessage(self.ledger_id, b"message"),
            ),
            SigningMessage(
                performative=SigningMessage.Performative.SIGNED_TRANSACTION,
                message_id=2,
                target=1,
                signed_transaction=SignedTransaction(
                    self.ledger_id, {"sig": "signature"}
                ),
            ),
            SigningMessage(
                performative=SigningMessage.Performative.SIGNED_MESSAGE,
                message_id=2,
                target=1,
                signed_message=SignedMessage(self.ledger_id, "message"),
            ),
            SigningMessage(
                performative=SigningMessage.Performative.ERROR,
                message_id=2,
                target=1,
                error_code=SigningMessage.ErrorCode.UNSUCCESSFUL_MESSAGE_SIGNING,
            ),
        ]

    def build_inconsistent(self) -> List[SigningMessage]:  # type: ignore[override]
        """Build inconsistent messages to be used for testing."""
        return [
            SigningMessage(
                performative=SigningMessage.Performative.SIGN_TRANSACTION,
                terms=self.terms,
            ),
            SigningMessage(
                performative=SigningMessage.Performative.SIGN_TRANSACTION,
                raw_transaction=RawTransaction(self.ledger_id, {"tx": "transaction"}),
            ),
            SigningMessage(
                performative=SigningMessage.Performative.ERROR,
                message_id=2,
                target=1,
            ),
        ]


class TestDialogues(BaseProtocolDialoguesTestCase):
    """Test dialogues."""

    MESSAGE_CLASS: Type[Message] = SigningMessage
    DIALOGUE_CLASS: Type[BaseDialogue] = BaseSigningDialogue
    DIALOGUES_CLASS: Type[Dialogues] = BaseSigningDialogues
    ROLE_FOR_THE_FIRST_MESSAGE = BaseSigningDialogue.Role.SKILL

    def make_message_content(self) -> dict:
        """Make a dict with message contruction content for dialogues.create."""
        return dict(
            performative=SigningMessage.Performative.SIGN_TRANSACTION,
            terms=Terms(
                ledger_id="ledger_id",
                sender_address="address1",
                counterparty_address="address2",
                amount_by_currency_id={"FET": -2},
                quantities_by_good_id={"good_id": 10},
                is_sender_payable_tx_fee=True,
                nonce="transaction nonce",
            ),
            raw_transaction=RawTransaction("ledger_id", {"tx": "transaction"}),
        )



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022-2023 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""Memory usage across the time."""
import itertools
import struct
import time
from typing import Any, List, Tuple, Union, cast

from aea_cli_benchmark.utils import get_mem_usage_in_mb  # noqa: I100
from aea_cli_benchmark.utils import PACKAGES_DIR
from aea_cli_benchmark.utils import make_agent as base_make_agent
from aea_cli_benchmark.utils import make_skill, wait_for_condition

from aea.aea import AEA
from aea.common import Address
from aea.configurations.base import ConnectionConfig
from aea.identity.base import Identity
from aea.protocols.base import Message, Protocol
from aea.protocols.dialogue.base import Dialogue
from aea.registries.resources import Resources
from aea.runner import AEARunner
from aea.skills.base import Handler

from packages.fetchai.connections.local.connection import (  # noqa: E402 # pylint: disable=C0413
    LocalNode,
    OEFLocalConnection,
)
from packages.valory.protocols.http.dialogues import HttpDialogue, HttpDialogues
from packages.valory.protocols.http.message import HttpMessage


class HttpPingPongHandler(Handler):
    """Dummy handler to handle messages."""

    SUPPORTED_PROTOCOL = HttpMessage.protocol_id

    def setup(self) -> None:
        """Noop setup."""
        # pylint: disable=attribute-defined-outside-init, unused-argument
        self.count: int = 0
        self.rtt_total_time: float = 0.0
        self.rtt_count: int = 0

        self.latency_total_time: float = 0.0
        self.latency_count: int = 0

        def role(m: Message, addr: Address) -> Dialogue.Role:
            return HttpDialogue.Role.CLIENT

        self.dialogues = HttpDialogues(
            self.context.agent_address, role_from_first_message=role
        )

    def teardown(self) -> None:
        """Noop teardown."""

    def handle(self, message: Message) -> None:
        """Handle incoming message."""
        self.count += 1
        message = cast(HttpMessage, message)
        dialogue = self.dialogues.update(message)
        if not dialogue:
            raise Exception("something goes wrong")
        rtt_ts, latency_ts = struct.unpack("dd", message.body)  # type: ignore
        if message.performative == HttpMessage.Performative.REQUEST:
            self.latency_total_time += time.time() - latency_ts
            self.latency_count += 1
            self.make_response(cast(HttpDialogue, dialogue), message)
        elif message.performative == HttpMessage.Performative.RESPONSE:
            self.rtt_total_time += time.time() - rtt_ts
            self.rtt_count += 1

            # got response, make another request to the same agent
            self.make_request(message.sender)

    def make_response(self, dialogue: HttpDialogue, message: HttpMessage) -> None:
        """Construct and send a response for message received."""
        response_message = dialogue.reply(
            target_message=message,
            performative=HttpMessage.Performative.RESPONSE,
            version=message.version,
            headers="",
            status_code=200,
            status_text="Success",
            body=message.body,
        )
        self.context.outbox.put_message(response_message)

    def make_request(self, recipient_addr: str) -> None:
        """Make initial http request."""
        message, _ = self.dialogues.create(
            recipient_addr,
            performative=HttpMessage.Performative.REQUEST,
            method="get",
            url="some url",
            headers="",
            version="",
            body=struct.pack("dd", time.time(), time.time()),
        )
        self.context.outbox.put_message(message)


def make_agent(*args: Any, **kwargs: Any) -> AEA:
    """Make agent with http protocol support."""
    aea = base_make_agent(*args, **kwargs)
    aea.resources.add_protocol(
        Protocol.from_dir(str(PACKAGES_DIR / "fetchai" / "protocols" / "http"))
    )
    return aea


def run(
    duration: int,
    runtime_mode: str,
    runner_mode: str,
    start_messages: int,
    num_of_agents: int,
) -> List[Tuple[str, Union[int, float]]]:
    """Test multiagent message exchange."""
    # pylint: disable=import-outside-toplevel,unused-import
    # import manually due to some lazy imports in decision_maker
    import aea.decision_maker.default  # noqa: F401

    local_node = LocalNode()
    local_node.start()
    agents = []
    skills = {}
    handler_name = "httpingpong"

    for i in range(num_of_agents):
        agent_name = f"agent{i}"
        identity = Identity(agent_name, address=agent_name, public_key="some_key")
        resources = Resources()
        connection = OEFLocalConnection(
            local_node,
            configuration=ConnectionConfig(
                connection_id=OEFLocalConnection.connection_id,
            ),
            identity=identity,
            data_dir="tmp",
        )
        resources.add_connection(connection)
        agent = make_agent(
            agent_name=agent_name,
            runtime_mode=runtime_mode,
            resources=resources,
            identity=identity,
        )
        skill = make_skill(agent, handlers={handler_name: HttpPingPongHandler})
        agent.resources.add_skill(skill)
        agents.append(agent)
        skills[agent_name] = skill

    runner = AEARunner(agents, runner_mode)
    runner.start(threaded=True)
    for agent_ in agents:
        wait_for_condition(lambda a=agent_: a.is_running, timeout=5)
    wait_for_condition(lambda: runner.is_running, timeout=5)
    time.sleep(1)

    for agent1, agent2 in itertools.permutations(agents, 2):
        for _ in range(int(start_messages)):
            cast(
                HttpPingPongHandler,
                skills[agent1.identity.address].handlers[handler_name],
            ).make_request(agent2.identity.address)
    time.sleep(duration)

    mem_usage = get_mem_usage_in_mb()
    local_node.stop()
    runner.stop(timeout=5)
    total_messages = sum(
        [
            cast(HttpPingPongHandler, skill.handlers[handler_name]).count
            for skill in skills.values()
        ]
    )
    rate = total_messages / duration

    rtt_total_time = sum(
        [
            cast(HttpPingPongHandler, skill.handlers[handler_name]).rtt_total_time
            for skill in skills.values()
        ]
    )
    rtt_count = sum(
        [
            cast(HttpPingPongHandler, skill.handlers[handler_name]).rtt_count
            for skill in skills.values()
        ]
    )

    if rtt_count == 0:
        rtt_count = -1

    latency_total_time = sum(
        [
            cast(HttpPingPongHandler, skill.handlers[handler_name]).latency_total_time
            for skill in skills.values()
        ]
    )
    latency_count = sum(
        [
            cast(HttpPingPongHandler, skill.handlers[handler_name]).latency_count
            for skill in skills.values()
        ]
    )

    if latency_count == 0:
        latency_count = -1

    return [
        ("Total Messages handled", total_messages),
        ("Messages rate(envelopes/second)", rate),
        ("Mem usage(Mb)", mem_usage),
        ("RTT (ms)", rtt_total_time / rtt_count),
        ("Latency (ms)", latency_total_time / latency_count),
    ]



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2022 Valory AG
#   Copyright 2018-2021 Fetch.AI Limited
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------
"""This module contains the tests for the rl_agent module of the gym skill."""
# pylint: skip-file

import copy
from unittest.mock import patch

from packages.fetchai.skills.gym.helpers import ProxyEnv
from packages.fetchai.skills.gym.rl_agent import GoodPriceModel
from packages.fetchai.skills.gym.tests.intermediate_class import GymTestCase


class TestPriceBandit(GymTestCase):
    """Test PriceBandit of gym."""

    def test_sample(self):
        """Test the sample method of PriceBandit class."""
        sample = self.price_bandit.sample()
        assert type(sample) == int

    def test_update(self):
        """Test the update method of the PriceBandit class."""
        # before
        assert self.price_bandit.beta_a == self.mocked_beta_a
        assert self.price_bandit.beta_b == self.mocked_beta_b

        # operation
        self.price_bandit.update(True)

        # after & before
        assert self.price_bandit.beta_a == self.mocked_beta_a + 1
        assert self.price_bandit.beta_b == self.mocked_beta_b

        # operation
        self.price_bandit.update(False)

        # after
        assert self.price_bandit.beta_a == self.mocked_beta_a + 1
        assert self.price_bandit.beta_b == self.mocked_beta_b + 1


class TestGoodPriceModel(GymTestCase):
    """Test GoodPriceModel of gym."""

    def test_update_i(self):
        """Test the update method of the GoodPriceModel class where outcome is True."""
        # setup
        outcome = True
        mocked_price = 46
        before_prices = copy.deepcopy(self.good_price_model.price_bandits)

        # operation
        self.good_price_model.update(outcome, mocked_price)

        # after
        assert (
            self.good_price_model.price_bandits[mocked_price].beta_a
            != before_prices[mocked_price].beta_a
            and self.good_price_model.price_bandits[mocked_price].beta_b
            == before_prices[mocked_price].beta_b
        )
        for price in range(self.mocked_bound + 1):
            if price != mocked_price:
                assert (
                    self.good_price_model.price_bandits[price].beta_a
                    == before_prices[price].beta_a
                    and self.good_price_model.price_bandits[price].beta_b
                    == before_prices[price].beta_b
                )

    def test_update_ii(self):
        """Test the update method of the GoodPriceModel class where outcome is False."""
        # setup
        outcome = False
        mocked_price = 12
        before_prices = copy.deepcopy(self.good_price_model.price_bandits)

        # operation
        self.good_price_model.update(outcome, mocked_price)

        # after
        assert (
            self.good_price_model.price_bandits[mocked_price].beta_a
            == before_prices[mocked_price].beta_a
            and self.good_price_model.price_bandits[mocked_price].beta_b
            != before_prices[mocked_price].beta_b
        )
        for price in range(self.mocked_bound + 1):
            if price != mocked_price:
                assert (
                    self.good_price_model.price_bandits[price].beta_a
                    == before_prices[price].beta_a
                    and self.good_price_model.price_bandits[price].beta_b
                    == before_prices[price].beta_b
                )

    def test_get_price_expectation(self):
        """Test the get_price_expectation method of GoodPriceModel class."""
        expectation = self.good_price_model.get_price_expectation()
        assert type(expectation) == int


class TestMyRLAgent(GymTestCase):
    """Test MyRLAgent of gym."""

    def test_fit(self):
        """Test the fit method of the MyRLAgent class."""
        # setup
        step_result = ("obs", "reward", "done", "info")

        # operation
        with patch.object(ProxyEnv, "reset") as mocked_reset:
            with patch.object(
                ProxyEnv, "step", return_value=step_result
            ) as mocked_step:
                with patch.object(ProxyEnv, "close") as mocked_close:
                    with patch.object(
                        GoodPriceModel, "get_price_expectation"
                    ) as mocked_price_exp:
                        with patch.object(GoodPriceModel, "update") as mocked_update:
                            with patch.object(self.logger, "log") as mock_logger:
                                self.my_rl_agent.fit(self.proxy_env, self.nb_steps)

        # after
        # fit
        mocked_reset.assert_called_once()

        # _pick_an_action
        mocked_price_exp.assert_called()
        assert mocked_price_exp.call_count == self.nb_steps

        # fit
        mocked_step.assert_called()
        assert mocked_step.call_count == self.nb_steps

        # _update_model
        mocked_update.assert_called()
        assert mocked_update.call_count == self.nb_steps

        # fit
        mock_logger.assert_called()
        mocked_close.assert_called()



# -*- coding: utf-8 -*-
# ------------------------------------------------------------------------------
#
#   Copyright 2023 fetchai
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#
# ------------------------------------------------------------------------------

"""Test dialogues module for state_update protocol."""

# pylint: disable=too-many-statements,too-many-locals,no-member,too-few-public-methods,redefined-builtin
from aea.test_tools.test_protocol import BaseProtocolDialoguesTestCase

from packages.fetchai.protocols.state_update.dialogues import (
    StateUpdateDialogue,
    StateUpdateDialogues,
)
from packages.fetchai.protocols.state_update.message import StateUpdateMessage


class TestDialoguesStateUpdate(BaseProtocolDialoguesTestCase):
    """Test for the 'state_update' protocol dialogues."""

    MESSAGE_CLASS = StateUpdateMessage

    DIALOGUE_CLASS = StateUpdateDialogue

    DIALOGUES_CLASS = StateUpdateDialogues

    ROLE_FOR_THE_FIRST_MESSAGE = StateUpdateDialogue.Role.DECISION_MAKER  # CHECK

    def make_message_content(self) -> dict:
        """Make a dict with message contruction content for dialogues.create."""
        return dict(
            performative=StateUpdateMessage.Performative.INITIALIZE,
            exchange_params_by_currency_id={"some str": 1.4},
            utility_params_by_good_id={"some str": 1.4},
            amount_by_currency_id={"some str": 12},
            quantities_by_good_id={"some str": 12},
        )
